/**********************************************************************
* HEADER_START                                                        *
*                                                                     *
*      Name:           PROG.C                                         *
*      Project:        Developper's HC08 Bootloader Master            *
*      Description:    Main programming routines file                 *
*      Platform:       Linux/Win32                                    *
*      Revision:       1.0                                            *
*      Date:           Apr 8 2002                                     *
*      Author:         Michal Hanak                                   *
*      Company:        Motorola SPS                                   *
*      Security:       General Business                               *
*                                                                     *
* =================================================================== *
* Copyright (c):      MOTOROLA Inc., 2001, All rights reserved.       *
*                                                                     *
* =================================================================== *
* THIS SOFTWARE IS PROVIDED BY MOTOROLA "AS IS" AND ANY               *
* EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE *
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR  *
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL MOTOROLA OR              *
* ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    *
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT        *
* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;        *
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)            *
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, *
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)       *
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED *
* OF THE POSSIBILITY OF SUCH DAMAGE.                                  *
* =================================================================== *
*
* HEADER_END
*/

#include "sysdep.h"
#include "hc08sprg.h"
#include "serial.h"

/////////////////////////////////////////////////
// bootloader commands

#define WR_DATA		'W'
#define RD_DATA		'R'
#define IDENT		'I'
#define QUIT		'Q'
#define ERASE		'E'
#define ACK			0xFC

/////////////////////////////////////////////////////////////////
// helper compare of unsigned 

static unsigned umin(unsigned a, unsigned b)
{
	return a < b ? a : b;
}

//////////////////////////////////////////////////////////////////
// helper read 16 bit number
// return -1=error; 0=rd timeout; 2=success (2 bytes read)

static int r16(unsigned* n)
{
	int ret;
	BYTE b[2];

	// we need 2 bytes (otherwise return -1 or 0)
	if((ret=rb(b, 2)) != 2)	
		return ret < 0 ? ret : 0;

	// first was HI, then LO
	*n = b[0] << 8;
	*n += b[1];
	return 2;
}

/////////////////////////////////////////////////////////////////////
// helper write 16 bit number
// return -1=error; 0=wr timeout; 2=success (2 bytes written)

static int w16(unsigned n)
{
	int ret;
	BYTE b[2];

	b[0] = (BYTE) ((n & 0xff00) >> 8);
	b[1] = (BYTE) (n & 0xff);
	if((ret=wb(b, 2)) != 2) 
		return ret < 0 ? -1 : 0;

	return 2;
}

//////////////////////////////////////////////////////////////
// wait for hc08 reset

// target send us ACK (0xfc) but in this time, it might be off-key up to factor 3
// so we might receive FF, FE, FC, F8, ..., 80, 0

static int could_be_ack(unsigned b)
{
	b |= 0xff00;
	while(!(b&1))
		b = b >> 1;

	return (b & 0xff) == 0xff ? 1 : 0;
}

int hook_reset(void)
{
	int ret;
	BYTE d;

	printf("Waiting for HC08 reset ACK...");
	fflush(stdout);

	// ignore timeouts
	while((ret=rb(&d, 1)) >= 0)
	{
		if(ret > 0)
		{
			printf("received 0x%02x ", d);
			if(could_be_ack(d))
			{
				if(d == ACK) printf("(good).\n");
				else printf("(off-key).\n");
				break;
			}
			else
			{
				printf("(ignoring).\n");
			}
		}
	}

	if(ret < 0)
	{
		printf("\n");
		fprintf(stderr, "Error receiving reset ACK.\n");
		return -1;		 
	}

	// enter interactive mode
	if(wb(&d, 1) <= 0)
	{
		fprintf(stderr, "Error sending locking ACK.\n");
		return -1;		 
	}

	// wait for finish
	flush_uart(FDRAIN, 0);

	// success
	return 1;
}

///////////////////////////////////////////////////////////////
// send breaks to calibrate target's oscillator

int calibrate_speed(void)
{
	BYTE b;
	int ret, cnt=0;

	for(;;)
	{
		// wait for ACK or timeout
		ret = rb(&b, 1);
		// read error
		if(ret < 0)
		{
			if(cnt) printf("\n");
			fprintf(stderr, "Error receiving calibrate ACK.\n");
			return -1;
		}
		// got something
		else if(ret > 0)
		{
			// it is wrong to receive anything other than ACK
			if(b != ACK)
			{
				if(cnt) printf("\n");
				fprintf(stderr, "Wrong calibrate response (0x%02x, must be 0x%02x). Target is not calibrated.\n", b, ACK);
				return -1;
			}
			// OK, exit calibration loop
			break;
		}
		
		// rx timeout ! send 1.04ms break pulse (again)
		if(send_break10() < 0) 
		{
			if(cnt) printf("\n");
			fprintf(stderr, "Can't send break pulse.\n");
			return -1;
		}

		// another break sent
		printf("\rCalibration break pulse sent. Count: %d ", ++cnt);
		fflush(stdout);
	};

	if(cnt) printf("\n");
	// success (got ACK)
	return 1;
}


///////////////////////////////////////////////////////////////
// simple return-on-error macros

#define R8(var) if(rb(var, 1) != 1) return -1
#define R16(var) if(r16(var) <= 0) return -1
#define W8(n) { BYTE _x=n; if(wb(&_x, 1) != 1) return -1; }
#define W16(n) if(w16(n) <= 0) return -1

//////////////////////////////////////////////////////////////
// get basic info about target board

int read_mcu_info(void)
{
	BYTE d;
	unsigned i;

	// initialize memory space
	memset(&ident, 0, sizeof(ident));

	// issue command
	flush_uart(FDISCARD, FDISCARD);
	W8(IDENT);

	// first byte is protocol version (ignored)
	R8(&d); bl_version = (int) d;

	// read identification structure
	R16(&ident.mem_start);
	R16(&ident.mem_end);
	R16(&ident.bl_tbl);
	R16(&ident.int_vect_tbl);
	R16(&ident.erblk);
	R16(&ident.wrblk);

	// read private data
	if(rb(&ident.priv_data, sizeof(ident.priv_data)) != sizeof(ident.priv_data))
		return -1;

	// read string
	i = 0;
	// read all name characters
	do
	{
		R8(&d);

		// but store up to our buffer len chars only
		if(i < sizeof(ident.targ_name)-1)
			ident.targ_name[i++] = d;

	// zero terminated
	} while(d);

	return 0;
}

//////////////////////////////////////////////////////////////
// read memory block, return number of bytes read or -1

int read_blk(unsigned adr, int len, BYTE *dest)
{
	if(len < 0 || len >= 0x100 || adr >= 0x10000)
		return -1;

	// issue command
	flush_uart(FDISCARD, FDISCARD);
	W8(RD_DATA);
	// and parameters
	W16(adr);
	W8(len);

	// block read
	if(rb(dest, len) != len)
		return -1;

	// return number of bytes read (==requested)
	return len;
}

//////////////////////////////////////////////////////////////
// masquerade vector table (work on memory image)
// the trick is to move any vector values from S19 file to 
// special jump table located at ident.app_vect_tbl

int setup_vect_tbl(void)
{
	unsigned src, dest;
	int any_hit = 0;

	// start of jump table
	dest = ident.bl_tbl;
	// we store private data first
	memcpy(&image.d[dest], ident.priv_data, sizeof(ident.priv_data));
	dest += sizeof(ident.priv_data);

	// and jump-table next
	for(src = 0xfffe; src >= ident.int_vect_tbl; src -= 2)
	{
		image.d[dest++] = 0xcc;	// jump instruction
		image.d[dest++] = image.d[src]; 
		image.d[dest++] = image.d[src+1];

		// is the vector valid ?
		if(image.f[src] && image.f[src+1])
			any_hit = 1;

		// original vector (from s19 image) is no longer valid
		image.f[src] = 0;
		image.f[src+1] = 0;
	}

	// write the whole system area, but only if at least one vector was valid in the S19
	if(any_hit)
		memset(&image.f[ident.bl_tbl], 2, dest - ident.bl_tbl);

	return 0;
}

//////////////////////////////////////////////////////////////
// erase single block around given address

int erase_blk(unsigned a)
{
	int ret;
	BYTE d;

	DBG1("\nerase_blk: 0x%04x\n", a);
	if(a >= 0x10000)
		return -1;

	// issue command
	flush_uart(FDISCARD, FDISCARD);
	W8(ERASE);
	// and parameter
	W16(a);

	// wait for ack
	while((ret = rb(&d, 1)) == 0)
		; // do nothing

	// it must be ACK character
	if(ret > 0 && d != ACK)
		ret = -1;

	// either -1 or 1
	return ret;
}

//////////////////////////////////////////////////////////////
// program single block

int prg_blk(unsigned a, int len)
{
	int ret;
	BYTE d;

	DBG2("\nprg_blk: 0x%04x-0x%04x\n", a, a+len-1);
	if(len < 0 || len >= 0x100 || a >= 0x10000) 
		return -1;

	// issue command
	flush_uart(FDISCARD, FDISCARD);
	W8(WR_DATA);
	// and parameters
	W16(a);
	W8(len);

	if(wb(&image.d[a], len) != len)
		return -1;

	// wait for ack
	while((ret = rb(&d, 1)) == 0)
		; // do nothing

	// must be ACK character
	if(ret > 0 && d != ACK)
		ret = -1;

	// either -1 or 1
	return ret;
}

//////////////////////////////////////////////////////////////
// program one continuous area and give user a progess 

static int prg_area(unsigned start, unsigned end)
{
	unsigned i, er, wr, er_next, wr_end, wr_next;
	unsigned wr_one, written=0, total=0;

	DBG2("prg_area: 0x%04x-0x%04x\n", start, end);

	// count valid bytes
	for(i=start; i<end; i++)
		if(image.f[i]) total++;

	// take start address as it is, but do further steps to erblk boundaries
	for(er=start; er<end; er=er_next)
	{
		// start of next erase block
		er_next = (er+ident.erblk) & ~(ident.erblk-1);

		// anything to program in this erase block ?
		wr = wr_end = er;
		for(i=er; i<er_next; i++)
		{
			// valid byte
			if(image.f[i])
			{
				if(!image.f[wr]) wr = i;
				wr_end = i+1;
			}
		}

		// never pass after end
		if(wr_end > end)
			wr_end = end;

		// wr is now pointing to first valid byte (within current erase block)
		// wr_end is now pointing after last valid byte (within current erase block)
		if(wr < wr_end)
		{
			printf("\rMemory programming: E 0x%04X %d%%", wr, 100*written/total);
			fflush(stdout); DBG("\n");

			// use the first valid-byte address
			if(erase_blk(wr) < 0) 
			{
				printf("\n");
				fprintf(stderr, "Can't erase block at address 0x%04x\n", wr);
				return -1;
			}

			for(/* original wr */; wr<wr_end; wr=wr_next)
			{
				// start of next write block
				wr_next = (wr+ident.wrblk) & ~(ident.wrblk-1);
				
				printf("\rMemory programming: W 0x%04X %d%%", wr, 100*written/total);

				fflush(stdout); DBG("\n");

				wr_one = umin(wr_end, wr_next)-wr;
				if(prg_blk(wr, wr_one) < 0) 
				{
					printf("\n");
					fprintf(stderr, "Can't program block at address 0x%04x\n", wr);
					return -1;
				}

				// the percentage-counting algorithm is not perfect, in some cases there might 
				// be more than 100% achieved (if S19 file has holes within erblks = rare case)
				if((written += wr_one) > total)
					written = total;
			}
		}
	}

	return 0;
}

//////////////////////////////////////////////////////////////
// program all

int	prg_mem(void)
{
	unsigned s,e;

	// find first and last valid byte
	for(s=0; s<0x10000; s++)
		if(image.f[s]) break;
	for(e=0x10000; e>s; e--)
		if(image.f[e-1]) break;

	if(prg_area(s, e) < 0)
		return -1;

	printf("\rMemory programmed:          100%%\n");
	return 0;
}

//////////////////////////////////////////////////////////////
// check if code goes to valid memory only

int	check_image()
{
	unsigned a;

	// before valid flash
	for(a=0; a<ident.mem_start; a++)
		if(image.f[a]==1) goto err_hit;		// 1 means valid code, 2 is system code

	// after valid flash
	for(a=ident.mem_end; a<0x10000; a++)
		if(image.f[a]==1) goto err_hit;

	// no error
	return 0;

err_hit:
	fprintf(stderr, "S19 image will not fit into available flash memory (at address 0x%04x).\n", a);
	return -1;
}

//////////////////////////////////////////////////////////////
// exit program mode, start application

int	unhook(void)
{
	W8(QUIT);
	return 0;
}

