;************************************************
;* MegaJolt Lite Junior Ignition Controller
;* Copyright 2003 Brent Picasso 				
;* brent@picasso.org http://www.picasso.org
;* Contains Code Derived from
;* Bruce Bowling and Al Grippo		
;* http://www.bgsoflex.com/mjl/mjl_edis.html				
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.

;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.

;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;Description:
;
;an Internal Combustion Engine ignition controller
;designed to work with the Ford EDIS ignition
;control module
; 
; Updates:
; 7/2/2004  - Added KPA factor table and lookup to correct errors in MAP sensor readings
; 6/18/2005 - Added linear interpolation for ignition map
; 			- Fixed incorrect User defined output triggering - was triggering 100 RPM off
;			- Cleaned up startup routine. Fixed incorrect triggering of outputs during cranking RPM
;			- Added soft rev limiter. Advance goes to zero degrees 100 RPM before configured hard rev limit.
;				At configured rev limit, external output is enabled, and external circuitry can disable the ignition coils.
;			- Fixed problem when write flash is peformed, goes into endless loop trying to process sending of data out of controller
;			- Enabled Watchdog (Computer Operating Properly)
;			- Changed default ignition map to something sane that would work for both normally aspirated and boosted applications.
;				
;************************************************


				XDEF Entry,main,TIMER_ROLL,TIMER0,DOPIP,SCI_TX_READY,SCI_RX_READY
				Include 'MC68HC908KX8.equ'

initCONFIG1: 	equ		%00001110     	;Init. for Low Power Consumption. Enable COP (Computer Operating Properly)
initCONFIG2: 	equ		%00101001	  	;Enable external oscillator
initStack:   	equ		RamLast       	;init. stack pointer to last RAM

SelectCH2AD: 	equ		%00000010	  	;initial ADSCR select and Read CH2
SelectCH3AD: 	equ		%00000011     	;initial ADSCR Select and Read CH3
initADICLK   	equ		%01010000     	;initial ADICLK = Bus/4, use bus frequency
Timerstop    	equ  	%00110010     	;Stop timer, reset of timer counter, prescale for 1 usec
Timergo      	equ 	%00000010     	;Enable timer, prescale of 4 for 1 usec increment, interrupts
SetPWM       	equ  	%00011010     	;PWM, clear on compare, toggle state on overflow
SetPWMNoTog  	equ  	%00011000     	;PWM, clear on compare, no toggle on overflow
SetOCInitH		equ  	%00000000     	; Clear output

SetSawTimer	 	equ		%01011000	  	;pwm; clear on compare; no toggle on overflow; interrupt enabled

RPM_BIN_UBOUND	equ		9				;the upper bound of the RPM bins
MAP_BIN_UBOUND	equ		9				;the upper bound of the MAP bins

;*************************************************
;communication command constants
;*************************************************
;current command Modes
GetState		equ		1				;Retrieve the current state
GetConfig	    equ		2				;Retrieve Ignition Controller Configuration
SetConfig	    equ		3				;Set the ignition Map Config
UpdateConfig	equ		4				;Update the Ignition Controller Configuration
WriteFlash		equ		5				;write Ignition controller configuration to flash

;command lengths
GetStateLength	equ		6				;data packet length of current runtime state
GetConfigLength equ		128				;data packet length of ignition controller configuration



DEFAULT_RAM SECTION SHORT    

;!!! Important! do not change order of data - the following needs to precede the next block of data!  see AN1831
MAPValue			ds      1				;value read from ADC
RPMValue			ds		1				;the actual RPM Value /100 (i.e. 5200 RPM = 52). in increments of 100 RPM
BelowIdleFlag		ds		1				;non zero indicates the engine is below idle (i.e. not running)
RPMCountHigh		ds		1				;RPM tick count high value
RPMCountLow			ds		1				;RPM tick count low value
AdvanceDegrees		ds		1				;current advance degrees

CurrentRPMBin		ds		1				;the selected RPM range bin offset
CurrentMAPBin		ds		1				;the selected MAP range bin offset

;!!! Important! do not change order of data - the following needs to start at RAM+$08!  see AN1831
CTRLBYT				ds		1				;Control byte for Flash programming routine
CPUSPD				ds		1				;CPU Speed flag
LADDR				ds		2				;last address for range program

;Ignition Configuration start-- Must start at RAM+$0C for proper Flash programming see AN1831
;!!! Important! do not change the order of this data!
CONFIG_DATA:
RPMBINS 			ds		10				;RPM bins- actual RPM value / 100 (i.e. 5200 RPM = 52). in increments of 100 RPM
MAPBINS				ds		10				;MAP bins- calculated MAP KPa values
IGNITION_MAP		ds		100				;10 x 10 map of ignition values, with actual ignition advance value- 0 to 59 degrees
CONFIG_CYLINDERS 	ds		1				;number of cylinders - 4,6 or 8

USER_OUTPUT_CONFIG	ds		1				;bits 7-4 indicate output type- 1 = RPM switch, 0 = MAP switch
											;bits 3-0 indicate output mode- 1 = "on" on exceed threshold, 0 = "off" on exceed threshold
											
USER_OUTPUT_1		ds		1				;user output switch 1 threshold value- either RPM value or MAP value, in the format of RPM/MAP bin values
USER_OUTPUT_2		ds		1				;user output switch 2 threshold value- either RPM value or MAP value, in the format of RPM/MAP bin values
USER_OUTPUT_3		ds		1				;user output switch 3 threshold value- either RPM value or MAP value, in the format of RPM/MAP bin values
USER_OUTPUT_4		ds		1				;user output switch 4 threshold value- either RPM value or MAP value, in the format of RPM/MAP bin values

REV_LIMITER			ds		1				;rev limiter output switch (contains RPM value in same format as in RPM Bins)
RPM_SHIFT_LIGHT		ds		1				;shift light output switch value (contains RPM value in same format as in RPM Bins)
;Ignition Configuration end

SAWActive			ds		1				;SAW signal is currently being processed
currentCommand		ds		1				;the current communication command that is active
cmdSendIndex		ds		1				;the current command index value when transmitting multiple bytes
											;i.e. tracks where we are during data transmit

interp_x1			ds		1				;interpolation X1 reference
interp_x2			ds		1				;interpolation X2 reference
interp_y1			ds		1				;interpolation Y1 reference
interp_y2			ds		1				;interpolation Y2 reference
interp_x			ds		1				;interpolation reference input
interp_y			ds		1				;interpolation result
interp_neg_slope	ds		1				;interpolation negative slope indicator

DEFAULT_ROM		SECTION


;**************************************************************
;* Main_Init - This is the point where code starts executing  *
;*             after a RESET.                                 *
;**************************************************************
Entry:
main:
		rsp
		clra
		clrx

		clra                 				;Init. Acc. For simulation
		mov    #initCONFIG1,CONFIG1 		;initialize Config1
		mov    #initCONFIG2,CONFIG2 		;initialize Config2

		mov		#initADICLK,ADCLK			;initialize ADC clock prescalar

		ldhx   #initStack+1  				;initialize the stack pointer
		txs

		jsr		InitExtOsc					;initialize external osciallator
		jsr		InitSerial					;initialize serial i/o
		jsr		InitPorts					;initialize ports

		ldhx   #RamStart     				;point to start of RAM


ClearRAM:
		clr    ,x            				;clear RAM location
		aix    #1            				;advance pointer
		cphx   #RamLast+1    				;done ?
		bne    ClearRAM      				;loop back if not


		jsr		UPDATE_RAM_CONFIG			;copy the saved configuration in flash to runtime location in RAM

		mov		#1,BelowIdleFlag			;we're below idle when we start up
		mov		#0,RPMValue
		jsr		READ_MAP					;perform an initial MAP reading
		jsr		UPDATE_USER_OUTPUTS			;initially set the user outputs
		jsr 	SET_ADVANCE_DEGREES			;set the initial degrees of advance
		
											;Set up the timer
        mov     #Timerstop,tsc        		;Stop Timer so it can be set up
        
		bclr	IMASK1,INTSCR   			; Enable IRQ
		bclr	MODE1,INTSCR				;set the interrupt mode for falling edge
		
		mov		#$FF,RPMCountHigh			;initialize RPM Count to minumum RPM value
		mov		#$FF,RPMCountLow			;initialize RPM Count to minimum RPM value

		
				
        cli									;enable interrupts
        


        
;***************************************************************************
; Main Loop
;***************************************************************************

MAIN_LOOP:
	    sta COPCTL		   			;service cop control register to prevent reset

		bsr READ_MAP 				;read and store the MAP value
       	bsr UPDATE_USER_OUTPUTS
       	
		jmp MAIN_LOOP

;***************************************************************************
; Read the MAP sensor through ADC channel 2
;***************************************************************************
READ_MAP:
        clrh
        mov    #SelectCH2AD,ADSCR 		;Select channel2 PTA4
ADREAD:
        brclr  7,ADSCR,ADREAD 			;Wait till read is complete

        lda    	ADR           			;Load A with Analog Data Register
        tax
        lda     KPAFACTOR,x				;Convert A/D read to Kilopascals
        sta		MAPValue					;Store the value for later use
        rts

;***************************************************************************
; Process the user outputs
;***************************************************************************
UPDATE_USER_OUTPUTS:

;User output 1
		clrh

		brset	0,USER_OUTPUT_CONFIG,TYPE_RPM_1
		lda		USER_OUTPUT_1			;test type MAP
		cmp		MAPValue
		bhi		USER_OUTPUT_1_OFF
		bra		USER_OUTPUT_1_ON		
TYPE_RPM_1:
		lda		USER_OUTPUT_1
		cmp		RPMValue
		bhi		USER_OUTPUT_1_OFF

USER_OUTPUT_1_ON:
		brset	4,USER_OUTPUT_CONFIG,USER_OUTPUT_1_HIGH ;test for mode configuration 1=high, 0=low
		bra		USER_OUTPUT_1_LOW
USER_OUTPUT_1_OFF:
		brset	4,USER_OUTPUT_CONFIG,USER_OUTPUT_1_LOW
USER_OUTPUT_1_HIGH:
		bset	0,PORTA
		bra		PROCESS_OUTPUT_2
USER_OUTPUT_1_LOW:
		bclr	0,PORTA


;User output 2
PROCESS_OUTPUT_2:
		brset	1,USER_OUTPUT_CONFIG,TYPE_RPM_2
		lda		USER_OUTPUT_2			;test type MAP
		cmp		MAPValue
		bhi		USER_OUTPUT_2_OFF
		bra		USER_OUTPUT_2_ON		
TYPE_RPM_2:
		lda		USER_OUTPUT_2
		cmp		RPMValue
		bhi		USER_OUTPUT_2_OFF

USER_OUTPUT_2_ON:
		brset	5,USER_OUTPUT_CONFIG,USER_OUTPUT_2_HIGH ;test for mode configuration 1=high, 0=low
		bra		USER_OUTPUT_2_LOW
USER_OUTPUT_2_OFF:
		brset	5,USER_OUTPUT_CONFIG,USER_OUTPUT_2_LOW
USER_OUTPUT_2_HIGH:
		bset	1,PORTA
		bra		PROCESS_OUTPUT_3
USER_OUTPUT_2_LOW:
		bclr	1,PORTA

;User Output 3
PROCESS_OUTPUT_3:
		brset	2,USER_OUTPUT_CONFIG,TYPE_RPM_3
		lda		USER_OUTPUT_3			;test type MAP
		cmp		MAPValue
		bhi		USER_OUTPUT_3_OFF
		bra		USER_OUTPUT_3_ON		
TYPE_RPM_3:
		lda		USER_OUTPUT_3
		cmp		RPMValue
		bhi		USER_OUTPUT_3_OFF

USER_OUTPUT_3_ON:
		brset	6,USER_OUTPUT_CONFIG,USER_OUTPUT_3_HIGH ;test for mode configuration 1=high, 0=low
		bra		USER_OUTPUT_3_LOW
USER_OUTPUT_3_OFF:
		brset	6,USER_OUTPUT_CONFIG,USER_OUTPUT_3_LOW
USER_OUTPUT_3_HIGH:
		bset	3,PORTA
		bra		PROCESS_OUTPUT_4
USER_OUTPUT_3_LOW:
		bclr	3,PORTA

;User Output 4
PROCESS_OUTPUT_4:
		brset	3,USER_OUTPUT_CONFIG,TYPE_RPM_4
		lda		USER_OUTPUT_4			;test type MAP
		cmp		MAPValue
		bhi		USER_OUTPUT_4_OFF
		bra		USER_OUTPUT_4_ON		
TYPE_RPM_4:
		lda		USER_OUTPUT_4
		cmp		RPMValue
		bhi		USER_OUTPUT_4_OFF

USER_OUTPUT_4_ON:
		brset	7,USER_OUTPUT_CONFIG,USER_OUTPUT_4_HIGH ;test for mode configuration 1=high, 0=low
		bra		USER_OUTPUT_4_LOW
USER_OUTPUT_4_OFF:
		brset	7,USER_OUTPUT_CONFIG,USER_OUTPUT_4_LOW
USER_OUTPUT_4_HIGH:
		bset	4,PORTA
		bra		PROCESS_REV_LIMIT
USER_OUTPUT_4_LOW:
		bclr	4,PORTA

;Rev limiting process
PROCESS_REV_LIMIT:
		lda		REV_LIMITER
		cmp		RPMValue
		bhi		REV_LIMIT_OFF
REV_LIMIT_ON:
		bset	0,PORTB		
		bra		PROCESS_SHIFT_LIGHT
REV_LIMIT_OFF:		
		bclr	0,PORTB
;shift Light RPM switch
PROCESS_SHIFT_LIGHT:
		lda		RPM_SHIFT_LIGHT
		cmp		RPMValue
		bhi		SHIFT_LIGHT_OFF
SHIFT_LIGHT_ON:
		bset	1,PORTB
		bra		PROCESS_OUTPUT_DONE
SHIFT_LIGHT_OFF:
		bclr	1,PORTB
PROCESS_OUTPUT_DONE:
		rts

;***************************************************************************
; Copy saved ignition config to RAM
;***************************************************************************
UPDATE_RAM_CONFIG:

		clrh
		ldx		#128
UPDATE_RAM_CONTINUE:
		decx
		lda		SAVED_CONFIG_DATA,x
		sta		CONFIG_DATA,x
		cpx		#0
		bne		UPDATE_RAM_CONTINUE
		rts

;***************************************************************************
; Process get config
; Processes the transmit of the current configuration
;***************************************************************************


PROCESS_GET_CONFIG:
		
		lda		cmdSendIndex
		cmp		#GetConfigLength		;are we at the end?
		blo		SEND_NEXT_CONFIG_BYTE
        bclr    TE,SCC2                 ; Disable Transmit
        bclr    SCTIE,SCC2              ; Disable transmit interrupt		
        clr		currentCommand			; finished with this command
        BRA		DONE_PROCESS_GET_CONFIG
SEND_NEXT_CONFIG_BYTE:
		tax
		lda		CONFIG_DATA,x			;load a config data value from memory
		sta		SCDR					;send it out the serial port
		inc		cmdSendIndex			;increment for the next byte
				     
        
DONE_PROCESS_GET_CONFIG:        
		rts
		
;***************************************************************************
; Process get state
; Processes the transmit of the current ignition state
;***************************************************************************
PROCESS_GET_STATE:
		
		lda		cmdSendIndex
		cmp		#$00					;send ignition advance
		bne		STATE_VALUE_1
		lda		AdvanceDegrees
		sta		SCDR
		bra		DONE_PROCESS_GET_STATE
		
STATE_VALUE_1:
		cmp		#$01
		bne		STATE_VALUE_2
		lda		RPMCountHigh			;send the RPM count high byte
		sta		SCDR
		bra		DONE_PROCESS_GET_STATE
		
STATE_VALUE_2:
		cmp		#$02
		bne		STATE_VALUE_3
		lda		RPMCountLow				;send the RPM count low byte
		sta		SCDR
		bra		DONE_PROCESS_GET_STATE
		
STATE_VALUE_3:
		cmp		#$03
		bne		STATE_VALUE_4
		lda		CurrentRPMBin			;send the current RPM Bin
		sta		SCDR
		bra		DONE_PROCESS_GET_STATE
		
STATE_VALUE_4:
		cmp		#$04
		bne		STATE_VALUE_5
		lda		CurrentMAPBin			;send the current MAP bin
		sta		SCDR
		bra		DONE_PROCESS_GET_STATE

STATE_VALUE_5:
		cmp		#$05
		bne		STATE_VALUE_6
		lda		MAPValue
		sta		SCDR
		bra		DONE_PROCESS_GET_STATE

STATE_VALUE_6:							; All done
        bclr    TE,SCC2                 ; Disable Transmit
        bclr    SCTIE,SCC2              ; Disable transmit interrupt
        clr		currentCommand			; finished with this command		

DONE_PROCESS_GET_STATE:
		inc		cmdSendIndex
		rts

SCI_TX_READY:
		pshh

		lda		SCS1					;clear the SCRF bit

		lda		currentCommand
		cmp		#GetState				;continue processing the GetState command
		bne		NEXT1		
		bsr		PROCESS_GET_STATE
		bra		DONE_TX

NEXT1:
		cmp		#GetConfig
		bne		NEXT2
		bsr		PROCESS_GET_CONFIG
		bra		DONE_TX

NEXT2:
DONE_TX:		
		pulh
		rti
		
SCI_RX_READY:
		pshh
		
		lda		SCS1					;Clear the SCRF Bit
		
		lda		currentCommand
		cmp		#UpdateConfig			;are we in the middle of receiving a config update?
		bne		CHECK_COMMAND_VALUE
		bsr		PROCESS_UPDATE_CONFIG
		bra		DONE_RX
		
CHECK_COMMAND_VALUE:		
		lda 	SCDR					;load the command value
		
		cmp		#83						;character 'S'
		beq		MODE_GET_STATE			;get the current state

		cmp		#67						;character 'C'
		beq		MODE_GET_CONFIG			;get the current config
		
		cmp		#85						;character 'U'
		beq		MODE_UPDATE_CONFIG
		
		cmp		#87						;character 'W'
		beq		MODE_WRITE_FLASH
		
		bra		DONE_RX		

MODE_GET_STATE:

		mov		#GetState,currentCommand	;We're processing the 'GetState' command
		clr		cmdSendIndex				;reset the progress counter
		bra		START_SEND		

MODE_GET_CONFIG:
		mov		#GetConfig,currentCommand	;we're processing the 'GetConfig' command
		clr		cmdSendIndex				;reset the progress counter
		bra		START_SEND

MODE_UPDATE_CONFIG:								;we're processing the 'UpdateConfig' command
		MOV		#UpdateConfig,currentCommand
		clr		cmdSendIndex
		bra		DONE_RX							;we're expecting more data to follow... (don't enable transmit interrupt)
		
MODE_WRITE_FLASH:
		mov		#WriteFlash,currentCommand
		clr		cmdSendIndex
		bsr		WRITE_FLASH_CONFIG
		bra		DONE_RX
		
START_SEND:
		bset	TE,SCC2						; Enable transmit
		bset	SCTIE,SCC2					; Enable transmit interrupt

DONE_RX:
		pulh
		rti
;***************************************************************************
; Process update configuration
;***************************************************************************
PROCESS_UPDATE_CONFIG:

		lda		SCDR						;get the next value
		ldx		cmdSendIndex
		sta		CONFIG_DATA,x
		incx
		stx		cmdSendIndex
		cpx		#GetConfigLength
		blo		DONE_PROCESS_UPDATE_CONFIG
		clr		currentCommand				;all done with data transfer

DONE_PROCESS_UPDATE_CONFIG:
		rts
		
;***************************************************************************
; Write configuration to flash
;***************************************************************************
	
WRITE_FLASH_CONFIG:							;disable all interrupts
		
		mov 	#16,CPUSPD
		bclr 	MASSBIT,CTRLBYT
		LDHX 	#SAVED_CONFIG_DATA			; starting address of data in flash
		jsr 	ERARNGE 					; erase the 64 byte page starting at SAVED_CONFIG_DATA


		mov 	#16,CPUSPD
		bclr 	MASSBIT,CTRLBYT
		LDHX 	#SAVED_CONFIG_DATA+64		; starting address of data in flash
		jsr 	ERARNGE 					; erase the 64 byte page starting at SAVED_CONFIG_DATA + 64 bytes

		ldhx	#SAVED_CONFIG_DATA_END-1	; program config data
		STHX	LADDR
		MOV		#16,CPUSPD
		ldhx	#SAVED_CONFIG_DATA
	    JSR 	PRGRNGE		
        											
											;enable interrupts
		rts
				
;***************************************************************************
; Initialize External Oscillator
;***************************************************************************
InitExtOsc:	 							;use when config2 = %001010X0
										;or config2 = %001110X0
										;or config2 = %000010X0
		lda 	#$13
oscWait:
		sta		COPCTL					; service COP register while waiting
								 		; turn on both ICG & ECG
								 		
		sta 	icgcr 
		cmpa	icgcr
		bne		oscWait

		rts

;***************************************************************************
; Initialize Ports
;***************************************************************************
InitPorts:

		bset	0,DDRA			;PTA0 output
		bclr	0,PORTA

		bset	1,DDRA			;PTA1 output
		bclr	1,PORTA

		bset	3,DDRA			;PTA3 output
		bclr	3,PORTA
		
		bset	4,DDRA			;PTA4 output
		bclr	4,PORTA
		
		bset	0,DDRB			;PTB0 output
		bclr	0,PORTB
		
		bset	1,DDRB			;PTB1 output
		bclr	1,PORTB		
		
		rts


;***************************************************************************
; Initialize SPI interface
;***************************************************************************
InitSerial:

		mov   #%00110000,SCBR       ; Baud Rate = 4800 - highest conventional baud rate using
									; a 16MHz clock input
									
		bset	ENSCI,SCC1		; Enable SCI
		bset	RE,SCC2			; Enable receiver
        bset    SCRIE,SCC2      ; Enable Receive interrupt
		lda		SCS1			; Clear SCI transmitter Empty Bit
			
		rts

;***************************************************************************
; SET_RPM_VALUE
;
; Looks up the RPM Value variable based on the RPMCountHigh and RPMCountLow values
; The RPM value is actual RPM/100 (i.e. 5200 RPM = 52) in increments of 100 RPM
; value is stored in variable RPMValue
;***************************************************************************
SET_RPM_VALUE:

		lda		BelowIdleFlag
		cmp		#1
		beq		ZERO_RPM
		
		ldhx	#96						;go backwards through the table starting at 10,000 RPM
		
		lda		CONFIG_CYLINDERS
		cmp		#4						;4 cylinder?
		beq		NEXT_RPM_TICK_4
		cmp		#6
		beq		NEXT_RPM_TICK_6
		bra		NEXT_RPM_TICK_8
				
NEXT_RPM_TICK_4:
		decx	
		beq		ZERO_RPM
		
		lda		RPM_TICKS_HIGH_4,X
		cmp		RPMCountHigh
		blo		NEXT_RPM_TICK_4
		bhi		RPM_VALUE_FOUND
		lda		RPM_TICKS_LOW_4,X
		cmp		RPMCountLow
		blo		NEXT_RPM_TICK_4
		bra		RPM_VALUE_FOUND

NEXT_RPM_TICK_6:
		decx
		beq		ZERO_RPM
		
		lda		RPM_TICKS_HIGH_6,X
		cmp		RPMCountHigh
		blo		NEXT_RPM_TICK_6
		bhi		RPM_VALUE_FOUND
		lda		RPM_TICKS_LOW_6,X
		cmp		RPMCountLow
		blo		NEXT_RPM_TICK_6
		bra		RPM_VALUE_FOUND
		
NEXT_RPM_TICK_8:
		decx
		beq		ZERO_RPM
		
		lda		RPM_TICKS_HIGH_8,X
		cmp		RPMCountHigh
		blo		NEXT_RPM_TICK_8
		bhi		RPM_VALUE_FOUND
		lda		RPM_TICKS_LOW_8,X
		cmp		RPMCountLow
		blo		NEXT_RPM_TICK_8
		bra		RPM_VALUE_FOUND
		
RPM_VALUE_FOUND:

		txa
		add		#5						;offset it back for starting at 500 RPM
		sta		RPMValue	
		rts
ZERO_RPM:
		mov		#0,RPMValue				;RPM is officially zero
		rts

;***************************************************************************
;***************************************************************************
;***************************************************************************
; PIP Command word receive - sets up timer with SAW pulsewidth
;***************************************************************************
;***************************************************************************
;***************************************************************************
DOPIP:
		pshh
		clrh
		sta		COPCTL					; poke watchdog
		
		bset	ACK1,INTSCR				; Flush out any new interrupts pending
		bih		DONE_DOPIP				;if the interrupt pin is still high, skip

		bset	IMASK1,INTSCR   		; Disable IRQ (until PWM timer is done)
		
										;If we're in a measurable RPM range (i.e. the timer hasn't rolled yet)
		lda		TCNTH					;save the current timer values as RPM count
		sta		RPMCountHigh
		lda 	TCNTL					 
		sta		RPMCountLow

        lda     tsc        
        MOV     #Timerstop,tsc        	; Stop Timer so it can be set up
		

		clrh
		ldx		AdvanceDegrees			;load the advance degrees from the previous calculation. Advance calculation is delayed by 1 ignition event
										;due to processing load

		lda		ADVANCE_HIGHB, X		;extract the advance value from the pre-calculated table,
		sta		TCH0H					;store it in the timer channel 0 output compare
		lda		ADVANCE_LOWB, X
		sta		TCH0L

        mov     #SetOCInitH,TSC0		;clear and
       	mov		#SetSawTimer,TSC0       ;set up the timer

        MOV     #Timergo,TSC          	; set timer prescaler to divide by 4 and start the SAW pulse!

        mov		#1,SAWActive


		jsr		SET_RPM_VALUE			;set the actual RPM Value based the RPM Count High/Low value
		jsr		SET_ADVANCE_DEGREES		;get the current advance degrees

DONE_DOPIP:		

		pulh
		rti


		

;***************************************************************************
; SET_ADVANCE_DEGREES
;
; Looks up the correct ignition advance value from the ignition table
; based on the target RPM and MAP value.
; See the "PROGRAMMABLE DATA AREA" section below for more information.
;***************************************************************************
SET_ADVANCE_DEGREES:

		lda		REV_LIMITER				;first do soft Rev limit check.
		deca							;Soft rev limit is REV_LIMITER-1 (Rev limit RPM - 100 RPM)
		cmp		RPMValue
		bhi		NO_SOFT_REV_LIMIT		
		mov		#0,AdvanceDegrees		;set advance to zero, soft rev limit enabled
		rts
		
NO_SOFT_REV_LIMIT:						;And continue...				
		ldhx 	#0						;Figure out what RPM bin to use.
TEST_RPM_BIN:							
		lda		RPMBINS,X				;walk through the RPM bins until we find the one that matches
		cmp		RPMValue
		bls		NEXT_RPM_BIN
		txa
		cmp		#0						;did we match to the first map bin?
		beq		USING_FIRST_RPM_BIN
		deca							;back up to the last one- that's the target
USING_FIRST_RPM_BIN:
		sta		CurrentRPMBin
		bra		FIND_RPM_BIN_DONE
NEXT_RPM_BIN:
		txa
		cmp		#RPM_BIN_UBOUND			;we use the last one if we've run out of MAP bins
		bhs		USE_LAST_RPM_BIN
		incx
		bra		TEST_RPM_BIN
USE_LAST_RPM_BIN:
		mov		#RPM_BIN_UBOUND,CurrentRPMBin
FIND_RPM_BIN_DONE:		

FIND_MAP_BIN:							;Determine what MAP bin to use.
		ldhx	#0
TEST_MAP_BIN:
		lda		MAPBINS,X				;walk through the MAP bins until we find the one that matches.
		cmp		MAPValue
		bls		NEXT_MAP_BIN
		txa
		cmp		#0						;did we match to the first MAP bin?
		beq		USING_FIRST_MAP_BIN
		deca							;back up to the last one- that's the target.
USING_FIRST_MAP_BIN:
		sta		CurrentMAPBin
		bra		FIND_MAP_BIN_DONE
NEXT_MAP_BIN:
		txa
		cmp		#MAP_BIN_UBOUND 		;we stay with the last one if we've run out of MAP bins.
		bhs		USE_LAST_MAP_BIN
		incx
		bra		TEST_MAP_BIN
USE_LAST_MAP_BIN:
		mov		#MAP_BIN_UBOUND,CurrentMAPBin
FIND_MAP_BIN_DONE:

;-----------------------------------------------------
; start the interpolation process. in order to calculate the interpolated
; ignition advance, we'll be working with 4 ignition advance values
; somewhere within the ignition table, based on the CurrentMAPBin and the CurrentRPMBin.
; The CurrentMAPBin and CurrentRPMBin correspond to the upper left hand corner of the example
; listed below:
;
;                                            R P M  A X I S
;  -----------------------------------------------------------------------------------------
;M |
;A |                        ----------------------------------------
;P |                        | MAP-LOW_RPM-LOW   | MAP-LOW_RPM_HIGH |
;  |                        |--------------------------------------|
;A |                        | MAP-HIGH_RPM-LOW  | MAP-HIGH_RPM_HIGH|
;X |                        ----------------------------------------
;I |
;S |
;  -----------------------------------------------------------------------------------------
;
; steps:
;
; 1. interpolate ignition advance between MAP-LOW_RPM-LOW and MAP-LOW_RPM_HIGH and save the result
; 2. interoplate ignition advance between MAP-HIGH_RPM-LOW and MAP-HIGH_RPM_HIGH and save the result
; 3. interpolate between the values generated from step 1 and 2 using the CurrentMAPValue for scaling
;    and store that as the final interpolated value
;

; perform Step 1

;fetch the MAP-LOW_RPM-LOW ignition advance value

		ldx		CurrentRPMBin			;parameters
		lda		CurrentMAPBin			;parameters
		bsr		GET_DEGREE_VALUE		;ignition advance degree result in accumulator
		sta		interp_y1
		
;fetch the MAP-LOW_RPM-HIGH ignition advance value
										
		lda		#1						;indicate the next RPM Bin, if available
		bsr		GET_ADJACENT_RPM_BIN	;result in Accumulator
		tax								;switch to X as parameter
		lda		CurrentMAPBin			;parameter
		bsr		GET_DEGREE_VALUE		;ignition advance degree result in accumulator
		sta		interp_y2		
		
		
		ldx		CurrentRPMBin			;fetch the X1, X2 reference values (RPM bin values)
		lda		RPMBINS,X
		sta		interp_x1
		
		lda		#1
		bsr		GET_ADJACENT_RPM_BIN	;get the next RPM BIN, if available
		tax
		lda		RPMBINS,X
		sta		interp_x2
		
		lda		RPMValue				;Actual RPM/100 
		sta		interp_x
		bsr		LININTERP				;all ready to go, do the linear interpolation
		lda		interp_y				;this is the result
		psha							;save it for later
		
;perform step 2

;fetch the MAP-HIGH_RPM-LOW ignition advance value
		lda		#1						;indicate next MAP Bin, if available
		bsr		GET_ADJACENT_MAP_BIN	;result in Accumulator
		ldx		CurrentRPMBin			;parameters
		bsr		GET_DEGREE_VALUE		;ignition advance degree result in accumulator
		sta		interp_y1
		
;fetch the MAP-HIGH_RPM-HIGH ignition advance value
		lda		#1						;indicate the next RPM bin, if available
		bsr		GET_ADJACENT_RPM_BIN	;result in Accumulator
		psha							;save it for later
		lda		#1						;indicate the next MAP bin, if available
		bsr		GET_ADJACENT_MAP_BIN	;result in Accumulator, input to parameter
		pulx							;pull back RPM Bin, input to parameter
		bsr		GET_DEGREE_VALUE		;ignition advance degree result in accumulator
		sta		interp_y2
		
		bsr		LININTERP				;all ready to go, do the linear interpolation
										;note that interp_X1 and interp_X2 do not need to change
		lda		interp_y

;perform step 3
		sta		interp_y2				;save result as the input to the next interpolation process

		pula	
		sta		interp_y1				;save step 1 result as the input to the next interpolation process
		
		ldx		CurrentMAPBin			;get the MAP value for the CurrentMAPBin
		lda		MAPBINS,X
		sta		interp_x1
		
		lda		#1						;get the MAP Bin value for the next MAP Bin, if available
		bsr		GET_ADJACENT_MAP_BIN
		tax
		lda		MAPBINS,X
		sta		interp_x2
		
		lda		MAPValue				;store the current MAP value as input to interpolation
		sta		interp_x
		
		bsr		LININTERP				;last interpolation call
		lda		interp_y
		sta		AdvanceDegrees			;store it as the current advance degrees

		rts

		
;***************************************************************
;** look up the target ignition advance value, based on the 
;** specified MAP bin and RPM bin
;
;** Parameters:
;** Accumulator has MAP Bin
;** X has RPM Bin
;** returns result in accumulator		
;***************************************************************
GET_DEGREE_VALUE:						
		
		pshx							;save the RPM Bin value
		ldhx	#RPM_BIN_UBOUND+1		;Y-axis map bin target * the RPM bin size
		mul								;skip down to the appropriate MAP Bin row in the ingition map
		
		add		1,SP					;add the RPM bin that was put on the Stack pointer
										;to index across the RPM (X axis) to the final target
		tax
		lda		IGNITION_MAP,X			;extract the igntion value
		
		pulx							;clean up
		rts								;result left in accumulator

;***************************************************************
;** find an adjacent RPM_BIN, if available. Returns CurrentRPMBin+X, if the RPMBin
;** isn't railed. if so, it just returns CurrentRPMBin
;**
;** Depends on CurrentRPMBin
;** Accumulator holds offset. e.g. 0 = returns CurrentRPMBin , 1=returns CurrentRPMBin+1, etc.
;** Target RPM Bin returned in accumulator
;***************************************************************
GET_ADJACENT_RPM_BIN:
		add		CurrentRPMBin
		cmp		#RPM_BIN_UBOUND
		bhs		CURRENT_RPM_BIN_RAILED
		rts
CURRENT_RPM_BIN_RAILED:
		lda		CurrentRPMBin
		rts
		
;***************************************************************
;** find an adjacent MAP_BIN, if available. Returns CurrentMAPBin+X, if the MAP Bin
;** isn't railed. if so, it returns CurrentMAPBin
;**
;** Depends on CurrentMAPBin
;** Accumulator holds offset. e.g. 0 = returns CurrentMAPBin , 1=returns  CurrentMAPBin+1, etc.
;** Target MAP Bin returned in accumulator
;***************************************************************
GET_ADJACENT_MAP_BIN:
		add		CurrentMAPBin
		cmp		#MAP_BIN_UBOUND
		bhs		CURRENT_MAP_BIN_RAILED
		rts
CURRENT_MAP_BIN_RAILED:
		lda		CurrentMAPBin
		rts


;***************************************************************************
;**
;** Linear Interpolation - 2D
;**
;**            (y2 - y1)
;**  y = y1 +  --------- * (x - x1)
;**            (x2 - x1)
;**
;**   interp_x1 = x1	(input low reference)
;**   interp_x2 = x2    (input high reference)
;**   interp_y1 = y1	(target low reference)
;**   interp_y2 = y2	(target high reference)
;**   interp_x = x		(input reference value)
;**   interp_y = y		(target interpolated result stored here)
;**	  interp_neg_slope (negative slope indicator)
;***************************************************************************
LININTERP:
        clr     interp_neg_slope    ; This is the negative slope detection bit
        mov     interp_y1,interp_y
CHECK_LESS_THAN:
        lda     interp_x
        cmp     interp_x1
        bhi     CHECK_GREATER_THAN
        bra     DONE_WITH_INTERP
CHECK_GREATER_THAN:
        lda     interp_x
        cmp     interp_x2
        blo     DO_INTERP
        mov     interp_y2,interp_y
        bra     DONE_WITH_INTERP

DO_INTERP:
        mov     interp_y1,interp_y
        lda     interp_x2
        sub     interp_x1
        beq     DONE_WITH_INTERP
        psha
        lda     interp_y2
        sub     interp_y1
	bcc	POSINTERP
        nega
        inc     interp_neg_slope
POSINTERP:
        psha
        lda     interp_x
        sub     interp_x1
        beq     ZERO_SLOPE
        pulx
        mul
        pshx
        pulh
        pulx

        div

        psha
        lda     interp_neg_slope
        bne     NEG_SLOPE
        pula
        add     interp_y1
        sta     interp_y
        bra     DONE_WITH_INTERP
NEG_SLOPE:
        pula
        sta     interp_neg_slope
        lda     interp_y1
        sub     interp_neg_slope
        sta     interp_y
        bra     DONE_WITH_INTERP
ZERO_SLOPE:
        pula            ;clean stack
        pula            ;clean stack
DONE_WITH_INTERP:
        rts














	
TIMER_ROLL:
		rti

        
TIMER0:
		pshh

		bset	ACK1,INTSCR				; Flush out any new interrupts pending
		bclr	IMASK1,INTSCR   		; Enable IRQ
		
		bclr	CH0F,TSC0
		
		lda		SAWActive
		cmp		#0
		bne		RESET_TIMER
	    mov		#1,BelowIdleFlag		;the timer count looped completely- we're running below idle
	    mov		#0,RPMValue				;RPM is officially zero
        MOV     #Timerstop,tsc        	; Stop Timer
		bra		TIMER0_EXIT
RESET_TIMER:							;SAW signal is finished- adjust registers so TIMER0
										;interrupt overflows at full value ($FFFF)
		clr		BelowIdleFlag
		lda		#0
		sta		TCH0H
		sta		TCH0L					;reset timer channel registers to full value	
		mov		#0,SAWActive
		mov		#Timergo,tsc			;resume the timer
			
TIMER0_EXIT:
		pulh
		rti
	
	
;*********************************************
;STATIC CONFIGURATION DATA
;*********************************************

;RPM tick count lookup, 4 Cylinder
RPM_TICKS_HIGH_4:
	DC.B	$EA		;500RPM High Byte, 4 cylinder
	DC.B	$C3		;600RPM High Byte, 4 cylinder
	DC.B	$A7		;700RPM High Byte, 4 cylinder
	DC.B	$92		;800RPM High Byte, 4 cylinder
	DC.B	$82		;900RPM High Byte, 4 cylinder
	DC.B	$75		;1000RPM High Byte, 4 cylinder
	DC.B	$6A		;1100RPM High Byte, 4 cylinder
	DC.B	$61		;1200RPM High Byte, 4 cylinder
	DC.B	$5A		;1300RPM High Byte, 4 cylinder
	DC.B	$53		;1400RPM High Byte, 4 cylinder
	DC.B	$4E		;1500RPM High Byte, 4 cylinder
	DC.B	$49		;1600RPM High Byte, 4 cylinder
	DC.B	$44		;1700RPM High Byte, 4 cylinder
	DC.B	$41		;1800RPM High Byte, 4 cylinder
	DC.B	$3D		;1900RPM High Byte, 4 cylinder
	DC.B	$3A		;2000RPM High Byte, 4 cylinder
	DC.B	$37		;2100RPM High Byte, 4 cylinder
	DC.B	$35		;2200RPM High Byte, 4 cylinder
	DC.B	$32		;2300RPM High Byte, 4 cylinder
	DC.B	$30		;2400RPM High Byte, 4 cylinder
	DC.B	$2E		;2500RPM High Byte, 4 cylinder
	DC.B	$2D		;2600RPM High Byte, 4 cylinder
	DC.B	$2B		;2700RPM High Byte, 4 cylinder
	DC.B	$29		;2800RPM High Byte, 4 cylinder
	DC.B	$28		;2900RPM High Byte, 4 cylinder
	DC.B	$27		;3000RPM High Byte, 4 cylinder
	DC.B	$25		;3100RPM High Byte, 4 cylinder
	DC.B	$24		;3200RPM High Byte, 4 cylinder
	DC.B	$23		;3300RPM High Byte, 4 cylinder
	DC.B	$22		;3400RPM High Byte, 4 cylinder
	DC.B	$21		;3500RPM High Byte, 4 cylinder
	DC.B	$20		;3600RPM High Byte, 4 cylinder
	DC.B	$1F		;3700RPM High Byte, 4 cylinder
	DC.B	$1E		;3800RPM High Byte, 4 cylinder
	DC.B	$1E		;3900RPM High Byte, 4 cylinder
	DC.B	$1D		;4000RPM High Byte, 4 cylinder
	DC.B	$1C		;4100RPM High Byte, 4 cylinder
	DC.B	$1B		;4200RPM High Byte, 4 cylinder
	DC.B	$1B		;4300RPM High Byte, 4 cylinder
	DC.B	$1A		;4400RPM High Byte, 4 cylinder
	DC.B	$1A		;4500RPM High Byte, 4 cylinder
	DC.B	$19		;4600RPM High Byte, 4 cylinder
	DC.B	$18		;4700RPM High Byte, 4 cylinder
	DC.B	$18		;4800RPM High Byte, 4 cylinder
	DC.B	$17		;4900RPM High Byte, 4 cylinder
	DC.B	$17		;5000RPM High Byte, 4 cylinder
	DC.B	$16		;5100RPM High Byte, 4 cylinder
	DC.B	$16		;5200RPM High Byte, 4 cylinder
	DC.B	$16		;5300RPM High Byte, 4 cylinder
	DC.B	$15		;5400RPM High Byte, 4 cylinder
	DC.B	$15		;5500RPM High Byte, 4 cylinder
	DC.B	$14		;5600RPM High Byte, 4 cylinder
	DC.B	$14		;5700RPM High Byte, 4 cylinder
	DC.B	$14		;5800RPM High Byte, 4 cylinder
	DC.B	$13		;5900RPM High Byte, 4 cylinder
	DC.B	$13		;6000RPM High Byte, 4 cylinder
	DC.B	$13		;6100RPM High Byte, 4 cylinder
	DC.B	$12		;6200RPM High Byte, 4 cylinder
	DC.B	$12		;6300RPM High Byte, 4 cylinder
	DC.B	$12		;6400RPM High Byte, 4 cylinder
	DC.B	$12		;6500RPM High Byte, 4 cylinder
	DC.B	$11		;6600RPM High Byte, 4 cylinder
	DC.B	$11		;6700RPM High Byte, 4 cylinder
	DC.B	$11		;6800RPM High Byte, 4 cylinder
	DC.B	$10		;6900RPM High Byte, 4 cylinder
	DC.B	$10		;7000RPM High Byte, 4 cylinder
	DC.B	$10		;7100RPM High Byte, 4 cylinder
	DC.B	$10		;7200RPM High Byte, 4 cylinder
	DC.B	$10		;7300RPM High Byte, 4 cylinder
	DC.B	$0F		;7400RPM High Byte, 4 cylinder
	DC.B	$0F		;7500RPM High Byte, 4 cylinder
	DC.B	$0F		;7600RPM High Byte, 4 cylinder
	DC.B	$0F		;7700RPM High Byte, 4 cylinder
	DC.B	$0F		;7800RPM High Byte, 4 cylinder
	DC.B	$0E		;7900RPM High Byte, 4 cylinder
	DC.B	$0E		;8000RPM High Byte, 4 cylinder
	DC.B	$0E		;8100RPM High Byte, 4 cylinder
	DC.B	$0E		;8200RPM High Byte, 4 cylinder
	DC.B	$0E		;8300RPM High Byte, 4 cylinder
	DC.B	$0D		;8400RPM High Byte, 4 cylinder
	DC.B	$0D		;8500RPM High Byte, 4 cylinder
	DC.B	$0D		;8600RPM High Byte, 4 cylinder
	DC.B	$0D		;8700RPM High Byte, 4 cylinder
	DC.B	$0D		;8800RPM High Byte, 4 cylinder
	DC.B	$0D		;8900RPM High Byte, 4 cylinder
	DC.B	$0D		;9000RPM High Byte, 4 cylinder
	DC.B	$0C		;9100RPM High Byte, 4 cylinder
	DC.B	$0C		;9200RPM High Byte, 4 cylinder
	DC.B	$0C		;9300RPM High Byte, 4 cylinder
	DC.B	$0C		;9400RPM High Byte, 4 cylinder
	DC.B	$0C		;9500RPM High Byte, 4 cylinder
	DC.B	$0C		;9600RPM High Byte, 4 cylinder
	DC.B	$0C		;9700RPM High Byte, 4 cylinder
	DC.B	$0B		;9800RPM High Byte, 4 cylinder
	DC.B	$0B		;9900RPM High Byte, 4 cylinder
	DC.B	$0B		;10000RPM High Byte, 4 cylinder
		
RPM_TICKS_LOW_4:
	DC.B	$60		;500RPM Low Byte, 4 cylinder
	DC.B	$50		;600RPM Low Byte, 4 cylinder
	DC.B	$69		;700RPM Low Byte, 4 cylinder
	DC.B	$7C		;800RPM Low Byte, 4 cylinder
	DC.B	$35		;900RPM Low Byte, 4 cylinder
	DC.B	$30		;1000RPM Low Byte, 4 cylinder
	DC.B	$89		;1100RPM Low Byte, 4 cylinder
	DC.B	$A8		;1200RPM Low Byte, 4 cylinder
	DC.B	$25		;1300RPM Low Byte, 4 cylinder
	DC.B	$B5		;1400RPM Low Byte, 4 cylinder
	DC.B	$20		;1500RPM Low Byte, 4 cylinder
	DC.B	$3E		;1600RPM Low Byte, 4 cylinder
	DC.B	$EF		;1700RPM Low Byte, 4 cylinder
	DC.B	$1B		;1800RPM Low Byte, 4 cylinder
	DC.B	$AD		;1900RPM Low Byte, 4 cylinder
	DC.B	$98		;2000RPM Low Byte, 4 cylinder
	DC.B	$CE		;2100RPM Low Byte, 4 cylinder
	DC.B	$44		;2200RPM Low Byte, 4 cylinder
	DC.B	$F3		;2300RPM Low Byte, 4 cylinder
	DC.B	$D4		;2400RPM Low Byte, 4 cylinder
	DC.B	$E0		;2500RPM Low Byte, 4 cylinder
	DC.B	$12		;2600RPM Low Byte, 4 cylinder
	DC.B	$67		;2700RPM Low Byte, 4 cylinder
	DC.B	$DA		;2800RPM Low Byte, 4 cylinder
	DC.B	$69		;2900RPM Low Byte, 4 cylinder
	DC.B	$10		;3000RPM Low Byte, 4 cylinder
	DC.B	$CD		;3100RPM Low Byte, 4 cylinder
	DC.B	$9F		;3200RPM Low Byte, 4 cylinder
	DC.B	$83		;3300RPM Low Byte, 4 cylinder
	DC.B	$78		;3400RPM Low Byte, 4 cylinder
	DC.B	$7B		;3500RPM Low Byte, 4 cylinder
	DC.B	$8D		;3600RPM Low Byte, 4 cylinder
	DC.B	$AC		;3700RPM Low Byte, 4 cylinder
	DC.B	$D7		;3800RPM Low Byte, 4 cylinder
	DC.B	$0C		;3900RPM Low Byte, 4 cylinder
	DC.B	$4C		;4000RPM Low Byte, 4 cylinder
	DC.B	$95		;4100RPM Low Byte, 4 cylinder
	DC.B	$E7		;4200RPM Low Byte, 4 cylinder
	DC.B	$41		;4300RPM Low Byte, 4 cylinder
	DC.B	$A2		;4400RPM Low Byte, 4 cylinder
	DC.B	$0B		;4500RPM Low Byte, 4 cylinder
	DC.B	$7A		;4600RPM Low Byte, 4 cylinder
	DC.B	$EF		;4700RPM Low Byte, 4 cylinder
	DC.B	$6A		;4800RPM Low Byte, 4 cylinder
	DC.B	$EA		;4900RPM Low Byte, 4 cylinder
	DC.B	$70		;5000RPM Low Byte, 4 cylinder
	DC.B	$FA		;5100RPM Low Byte, 4 cylinder
	DC.B	$89		;5200RPM Low Byte, 4 cylinder
	DC.B	$1C		;5300RPM Low Byte, 4 cylinder
	DC.B	$B4		;5400RPM Low Byte, 4 cylinder
	DC.B	$4F		;5500RPM Low Byte, 4 cylinder
	DC.B	$ED		;5600RPM Low Byte, 4 cylinder
	DC.B	$8F		;5700RPM Low Byte, 4 cylinder
	DC.B	$34		;5800RPM Low Byte, 4 cylinder
	DC.B	$DD		;5900RPM Low Byte, 4 cylinder
	DC.B	$88		;6000RPM Low Byte, 4 cylinder
	DC.B	$36		;6100RPM Low Byte, 4 cylinder
	DC.B	$E7		;6200RPM Low Byte, 4 cylinder
	DC.B	$9A		;6300RPM Low Byte, 4 cylinder
	DC.B	$50		;6400RPM Low Byte, 4 cylinder
	DC.B	$07		;6500RPM Low Byte, 4 cylinder
	DC.B	$C1		;6600RPM Low Byte, 4 cylinder
	DC.B	$7E		;6700RPM Low Byte, 4 cylinder
	DC.B	$3C		;6800RPM Low Byte, 4 cylinder
	DC.B	$FC		;6900RPM Low Byte, 4 cylinder
	DC.B	$BE		;7000RPM Low Byte, 4 cylinder
	DC.B	$81		;7100RPM Low Byte, 4 cylinder
	DC.B	$47		;7200RPM Low Byte, 4 cylinder
	DC.B	$0E		;7300RPM Low Byte, 4 cylinder
	DC.B	$D6		;7400RPM Low Byte, 4 cylinder
	DC.B	$A0		;7500RPM Low Byte, 4 cylinder
	DC.B	$6B		;7600RPM Low Byte, 4 cylinder
	DC.B	$38		;7700RPM Low Byte, 4 cylinder
	DC.B	$06		;7800RPM Low Byte, 4 cylinder
	DC.B	$D5		;7900RPM Low Byte, 4 cylinder
	DC.B	$A6		;8000RPM Low Byte, 4 cylinder
	DC.B	$78		;8100RPM Low Byte, 4 cylinder
	DC.B	$4B		;8200RPM Low Byte, 4 cylinder
	DC.B	$1E		;8300RPM Low Byte, 4 cylinder
	DC.B	$F3		;8400RPM Low Byte, 4 cylinder
	DC.B	$C9		;8500RPM Low Byte, 4 cylinder
	DC.B	$A0		;8600RPM Low Byte, 4 cylinder
	DC.B	$78		;8700RPM Low Byte, 4 cylinder
	DC.B	$51		;8800RPM Low Byte, 4 cylinder
	DC.B	$2B		;8900RPM Low Byte, 4 cylinder
	DC.B	$05		;9000RPM Low Byte, 4 cylinder
	DC.B	$E1		;9100RPM Low Byte, 4 cylinder
	DC.B	$BD		;9200RPM Low Byte, 4 cylinder
	DC.B	$9A		;9300RPM Low Byte, 4 cylinder
	DC.B	$77		;9400RPM Low Byte, 4 cylinder
	DC.B	$56		;9500RPM Low Byte, 4 cylinder
	DC.B	$35		;9600RPM Low Byte, 4 cylinder
	DC.B	$15		;9700RPM Low Byte, 4 cylinder
	DC.B	$F5		;9800RPM Low Byte, 4 cylinder
	DC.B	$D6		;9900RPM Low Byte, 4 cylinder
	DC.B	$B8		;10000RPM Low Byte, 4 cylinder	
	
;RPM tick count lookup, 6 Cylinder	
RPM_TICKS_HIGH_6:
	DC.B	$9C		;500RPM High Byte, 6 cylinder
	DC.B	$82		;600RPM High Byte, 6 cylinder
	DC.B	$6F		;700RPM High Byte, 6 cylinder
	DC.B	$61		;800RPM High Byte, 6 cylinder
	DC.B	$56		;900RPM High Byte, 6 cylinder
	DC.B	$4E		;1000RPM High Byte, 6 cylinder
	DC.B	$47		;1100RPM High Byte, 6 cylinder
	DC.B	$41		;1200RPM High Byte, 6 cylinder
	DC.B	$3C		;1300RPM High Byte, 6 cylinder
	DC.B	$37		;1400RPM High Byte, 6 cylinder
	DC.B	$34		;1500RPM High Byte, 6 cylinder
	DC.B	$30		;1600RPM High Byte, 6 cylinder
	DC.B	$2D		;1700RPM High Byte, 6 cylinder
	DC.B	$2B		;1800RPM High Byte, 6 cylinder
	DC.B	$29		;1900RPM High Byte, 6 cylinder
	DC.B	$27		;2000RPM High Byte, 6 cylinder
	DC.B	$25		;2100RPM High Byte, 6 cylinder
	DC.B	$23		;2200RPM High Byte, 6 cylinder
	DC.B	$21		;2300RPM High Byte, 6 cylinder
	DC.B	$20		;2400RPM High Byte, 6 cylinder
	DC.B	$1F		;2500RPM High Byte, 6 cylinder
	DC.B	$1E		;2600RPM High Byte, 6 cylinder
	DC.B	$1C		;2700RPM High Byte, 6 cylinder
	DC.B	$1B		;2800RPM High Byte, 6 cylinder
	DC.B	$1A		;2900RPM High Byte, 6 cylinder
	DC.B	$1A		;3000RPM High Byte, 6 cylinder
	DC.B	$19		;3100RPM High Byte, 6 cylinder
	DC.B	$18		;3200RPM High Byte, 6 cylinder
	DC.B	$17		;3300RPM High Byte, 6 cylinder
	DC.B	$16		;3400RPM High Byte, 6 cylinder
	DC.B	$16		;3500RPM High Byte, 6 cylinder
	DC.B	$15		;3600RPM High Byte, 6 cylinder
	DC.B	$15		;3700RPM High Byte, 6 cylinder
	DC.B	$14		;3800RPM High Byte, 6 cylinder
	DC.B	$14		;3900RPM High Byte, 6 cylinder
	DC.B	$13		;4000RPM High Byte, 6 cylinder
	DC.B	$13		;4100RPM High Byte, 6 cylinder
	DC.B	$12		;4200RPM High Byte, 6 cylinder
	DC.B	$12		;4300RPM High Byte, 6 cylinder
	DC.B	$11		;4400RPM High Byte, 6 cylinder
	DC.B	$11		;4500RPM High Byte, 6 cylinder
	DC.B	$10		;4600RPM High Byte, 6 cylinder
	DC.B	$10		;4700RPM High Byte, 6 cylinder
	DC.B	$10		;4800RPM High Byte, 6 cylinder
	DC.B	$0F		;4900RPM High Byte, 6 cylinder
	DC.B	$0F		;5000RPM High Byte, 6 cylinder
	DC.B	$0F		;5100RPM High Byte, 6 cylinder
	DC.B	$0F		;5200RPM High Byte, 6 cylinder
	DC.B	$0E		;5300RPM High Byte, 6 cylinder
	DC.B	$0E		;5400RPM High Byte, 6 cylinder
	DC.B	$0E		;5500RPM High Byte, 6 cylinder
	DC.B	$0D		;5600RPM High Byte, 6 cylinder
	DC.B	$0D		;5700RPM High Byte, 6 cylinder
	DC.B	$0D		;5800RPM High Byte, 6 cylinder
	DC.B	$0D		;5900RPM High Byte, 6 cylinder
	DC.B	$0D		;6000RPM High Byte, 6 cylinder
	DC.B	$0C		;6100RPM High Byte, 6 cylinder
	DC.B	$0C		;6200RPM High Byte, 6 cylinder
	DC.B	$0C		;6300RPM High Byte, 6 cylinder
	DC.B	$0C		;6400RPM High Byte, 6 cylinder
	DC.B	$0C		;6500RPM High Byte, 6 cylinder
	DC.B	$0B		;6600RPM High Byte, 6 cylinder
	DC.B	$0B		;6700RPM High Byte, 6 cylinder
	DC.B	$0B		;6800RPM High Byte, 6 cylinder
	DC.B	$0B		;6900RPM High Byte, 6 cylinder
	DC.B	$0B		;7000RPM High Byte, 6 cylinder
	DC.B	$0B		;7100RPM High Byte, 6 cylinder
	DC.B	$0A		;7200RPM High Byte, 6 cylinder
	DC.B	$0A		;7300RPM High Byte, 6 cylinder
	DC.B	$0A		;7400RPM High Byte, 6 cylinder
	DC.B	$0A		;7500RPM High Byte, 6 cylinder
	DC.B	$0A		;7600RPM High Byte, 6 cylinder
	DC.B	$0A		;7700RPM High Byte, 6 cylinder
	DC.B	$0A		;7800RPM High Byte, 6 cylinder
	DC.B	$09		;7900RPM High Byte, 6 cylinder
	DC.B	$09		;8000RPM High Byte, 6 cylinder
	DC.B	$09		;8100RPM High Byte, 6 cylinder
	DC.B	$09		;8200RPM High Byte, 6 cylinder
	DC.B	$09		;8300RPM High Byte, 6 cylinder
	DC.B	$09		;8400RPM High Byte, 6 cylinder
	DC.B	$09		;8500RPM High Byte, 6 cylinder
	DC.B	$09		;8600RPM High Byte, 6 cylinder
	DC.B	$08		;8700RPM High Byte, 6 cylinder
	DC.B	$08		;8800RPM High Byte, 6 cylinder
	DC.B	$08		;8900RPM High Byte, 6 cylinder
	DC.B	$08		;9000RPM High Byte, 6 cylinder
	DC.B	$08		;9100RPM High Byte, 6 cylinder
	DC.B	$08		;9200RPM High Byte, 6 cylinder
	DC.B	$08		;9300RPM High Byte, 6 cylinder
	DC.B	$08		;9400RPM High Byte, 6 cylinder
	DC.B	$08		;9500RPM High Byte, 6 cylinder
	DC.B	$08		;9600RPM High Byte, 6 cylinder
	DC.B	$08		;9700RPM High Byte, 6 cylinder
	DC.B	$07		;9800RPM High Byte, 6 cylinder
	DC.B	$07		;9900RPM High Byte, 6 cylinder
	DC.B	$07		;10000RPM High Byte, 6 cylinder
		
RPM_TICKS_LOW_6:
	DC.B	$40		;500RPM Low Byte, 6 cylinder
	DC.B	$35		;600RPM Low Byte, 6 cylinder
	DC.B	$9B		;700RPM Low Byte, 6 cylinder
	DC.B	$A8		;800RPM Low Byte, 6 cylinder
	DC.B	$CE		;900RPM Low Byte, 6 cylinder
	DC.B	$20		;1000RPM Low Byte, 6 cylinder
	DC.B	$06		;1100RPM Low Byte, 6 cylinder
	DC.B	$1B		;1200RPM Low Byte, 6 cylinder
	DC.B	$19		;1300RPM Low Byte, 6 cylinder
	DC.B	$CE		;1400RPM Low Byte, 6 cylinder
	DC.B	$15		;1500RPM Low Byte, 6 cylinder
	DC.B	$D4		;1600RPM Low Byte, 6 cylinder
	DC.B	$F5		;1700RPM Low Byte, 6 cylinder
	DC.B	$67		;1800RPM Low Byte, 6 cylinder
	DC.B	$1E		;1900RPM Low Byte, 6 cylinder
	DC.B	$10		;2000RPM Low Byte, 6 cylinder
	DC.B	$34		;2100RPM Low Byte, 6 cylinder
	DC.B	$83		;2200RPM Low Byte, 6 cylinder
	DC.B	$F8		;2300RPM Low Byte, 6 cylinder
	DC.B	$8D		;2400RPM Low Byte, 6 cylinder
	DC.B	$40		;2500RPM Low Byte, 6 cylinder
	DC.B	$0C		;2600RPM Low Byte, 6 cylinder
	DC.B	$EF		;2700RPM Low Byte, 6 cylinder
	DC.B	$E7		;2800RPM Low Byte, 6 cylinder
	DC.B	$F1		;2900RPM Low Byte, 6 cylinder
	DC.B	$0B		;3000RPM Low Byte, 6 cylinder
	DC.B	$34		;3100RPM Low Byte, 6 cylinder
	DC.B	$6A		;3200RPM Low Byte, 6 cylinder
	DC.B	$AD		;3300RPM Low Byte, 6 cylinder
	DC.B	$FA		;3400RPM Low Byte, 6 cylinder
	DC.B	$52		;3500RPM Low Byte, 6 cylinder
	DC.B	$B4		;3600RPM Low Byte, 6 cylinder
	DC.B	$1D		;3700RPM Low Byte, 6 cylinder
	DC.B	$8F		;3800RPM Low Byte, 6 cylinder
	DC.B	$08		;3900RPM Low Byte, 6 cylinder
	DC.B	$88		;4000RPM Low Byte, 6 cylinder
	DC.B	$0E		;4100RPM Low Byte, 6 cylinder
	DC.B	$9A		;4200RPM Low Byte, 6 cylinder
	DC.B	$2B		;4300RPM Low Byte, 6 cylinder
	DC.B	$C1		;4400RPM Low Byte, 6 cylinder
	DC.B	$5C		;4500RPM Low Byte, 6 cylinder
	DC.B	$FC		;4600RPM Low Byte, 6 cylinder
	DC.B	$9F		;4700RPM Low Byte, 6 cylinder
	DC.B	$47		;4800RPM Low Byte, 6 cylinder
	DC.B	$F2		;4900RPM Low Byte, 6 cylinder
	DC.B	$A0		;5000RPM Low Byte, 6 cylinder
	DC.B	$52		;5100RPM Low Byte, 6 cylinder
	DC.B	$06		;5200RPM Low Byte, 6 cylinder
	DC.B	$BE		;5300RPM Low Byte, 6 cylinder
	DC.B	$78		;5400RPM Low Byte, 6 cylinder
	DC.B	$34		;5500RPM Low Byte, 6 cylinder
	DC.B	$F3		;5600RPM Low Byte, 6 cylinder
	DC.B	$B5		;5700RPM Low Byte, 6 cylinder
	DC.B	$78		;5800RPM Low Byte, 6 cylinder
	DC.B	$3E		;5900RPM Low Byte, 6 cylinder
	DC.B	$05		;6000RPM Low Byte, 6 cylinder
	DC.B	$CF		;6100RPM Low Byte, 6 cylinder
	DC.B	$9A		;6200RPM Low Byte, 6 cylinder
	DC.B	$67		;6300RPM Low Byte, 6 cylinder
	DC.B	$35		;6400RPM Low Byte, 6 cylinder
	DC.B	$05		;6500RPM Low Byte, 6 cylinder
	DC.B	$D6		;6600RPM Low Byte, 6 cylinder
	DC.B	$A9		;6700RPM Low Byte, 6 cylinder
	DC.B	$7D		;6800RPM Low Byte, 6 cylinder
	DC.B	$53		;6900RPM Low Byte, 6 cylinder
	DC.B	$29		;7000RPM Low Byte, 6 cylinder
	DC.B	$01		;7100RPM Low Byte, 6 cylinder
	DC.B	$DA		;7200RPM Low Byte, 6 cylinder
	DC.B	$B4		;7300RPM Low Byte, 6 cylinder
	DC.B	$8F		;7400RPM Low Byte, 6 cylinder
	DC.B	$6B		;7500RPM Low Byte, 6 cylinder
	DC.B	$48		;7600RPM Low Byte, 6 cylinder
	DC.B	$25		;7700RPM Low Byte, 6 cylinder
	DC.B	$04		;7800RPM Low Byte, 6 cylinder
	DC.B	$E4		;7900RPM Low Byte, 6 cylinder
	DC.B	$C4		;8000RPM Low Byte, 6 cylinder
	DC.B	$A5		;8100RPM Low Byte, 6 cylinder
	DC.B	$87		;8200RPM Low Byte, 6 cylinder
	DC.B	$6A		;8300RPM Low Byte, 6 cylinder
	DC.B	$4D		;8400RPM Low Byte, 6 cylinder
	DC.B	$31		;8500RPM Low Byte, 6 cylinder
	DC.B	$16		;8600RPM Low Byte, 6 cylinder
	DC.B	$FB		;8700RPM Low Byte, 6 cylinder
	DC.B	$E1		;8800RPM Low Byte, 6 cylinder
	DC.B	$C7		;8900RPM Low Byte, 6 cylinder
	DC.B	$AE		;9000RPM Low Byte, 6 cylinder
	DC.B	$96		;9100RPM Low Byte, 6 cylinder
	DC.B	$7E		;9200RPM Low Byte, 6 cylinder
	DC.B	$67		;9300RPM Low Byte, 6 cylinder
	DC.B	$50		;9400RPM Low Byte, 6 cylinder
	DC.B	$39		;9500RPM Low Byte, 6 cylinder
	DC.B	$23		;9600RPM Low Byte, 6 cylinder
	DC.B	$0E		;9700RPM Low Byte, 6 cylinder
	DC.B	$F9		;9800RPM Low Byte, 6 cylinder
	DC.B	$E4		;9900RPM Low Byte, 6 cylinder
	DC.B	$D0		;10000RPM Low Byte, 6 cylinder
	
;RPM tick count lookup, 8 Cylinder	
RPM_TICKS_HIGH_8:
	DC.B	$75		;500RPM High Byte, 8 cylinder
	DC.B	$61		;600RPM High Byte, 8 cylinder
	DC.B	$53		;700RPM High Byte, 8 cylinder
	DC.B	$49		;800RPM High Byte, 8 cylinder
	DC.B	$41		;900RPM High Byte, 8 cylinder
	DC.B	$3A		;1000RPM High Byte, 8 cylinder
	DC.B	$35		;1100RPM High Byte, 8 cylinder
	DC.B	$30		;1200RPM High Byte, 8 cylinder
	DC.B	$2D		;1300RPM High Byte, 8 cylinder
	DC.B	$29		;1400RPM High Byte, 8 cylinder
	DC.B	$27		;1500RPM High Byte, 8 cylinder
	DC.B	$24		;1600RPM High Byte, 8 cylinder
	DC.B	$22		;1700RPM High Byte, 8 cylinder
	DC.B	$20		;1800RPM High Byte, 8 cylinder
	DC.B	$1E		;1900RPM High Byte, 8 cylinder
	DC.B	$1D		;2000RPM High Byte, 8 cylinder
	DC.B	$1B		;2100RPM High Byte, 8 cylinder
	DC.B	$1A		;2200RPM High Byte, 8 cylinder
	DC.B	$19		;2300RPM High Byte, 8 cylinder
	DC.B	$18		;2400RPM High Byte, 8 cylinder
	DC.B	$17		;2500RPM High Byte, 8 cylinder
	DC.B	$16		;2600RPM High Byte, 8 cylinder
	DC.B	$15		;2700RPM High Byte, 8 cylinder
	DC.B	$14		;2800RPM High Byte, 8 cylinder
	DC.B	$14		;2900RPM High Byte, 8 cylinder
	DC.B	$13		;3000RPM High Byte, 8 cylinder
	DC.B	$12		;3100RPM High Byte, 8 cylinder
	DC.B	$12		;3200RPM High Byte, 8 cylinder
	DC.B	$11		;3300RPM High Byte, 8 cylinder
	DC.B	$11		;3400RPM High Byte, 8 cylinder
	DC.B	$10		;3500RPM High Byte, 8 cylinder
	DC.B	$10		;3600RPM High Byte, 8 cylinder
	DC.B	$0F		;3700RPM High Byte, 8 cylinder
	DC.B	$0F		;3800RPM High Byte, 8 cylinder
	DC.B	$0F		;3900RPM High Byte, 8 cylinder
	DC.B	$0E		;4000RPM High Byte, 8 cylinder
	DC.B	$0E		;4100RPM High Byte, 8 cylinder
	DC.B	$0D		;4200RPM High Byte, 8 cylinder
	DC.B	$0D		;4300RPM High Byte, 8 cylinder
	DC.B	$0D		;4400RPM High Byte, 8 cylinder
	DC.B	$0D		;4500RPM High Byte, 8 cylinder
	DC.B	$0C		;4600RPM High Byte, 8 cylinder
	DC.B	$0C		;4700RPM High Byte, 8 cylinder
	DC.B	$0C		;4800RPM High Byte, 8 cylinder
	DC.B	$0B		;4900RPM High Byte, 8 cylinder
	DC.B	$0B		;5000RPM High Byte, 8 cylinder
	DC.B	$0B		;5100RPM High Byte, 8 cylinder
	DC.B	$0B		;5200RPM High Byte, 8 cylinder
	DC.B	$0B		;5300RPM High Byte, 8 cylinder
	DC.B	$0A		;5400RPM High Byte, 8 cylinder
	DC.B	$0A		;5500RPM High Byte, 8 cylinder
	DC.B	$0A		;5600RPM High Byte, 8 cylinder
	DC.B	$0A		;5700RPM High Byte, 8 cylinder
	DC.B	$0A		;5800RPM High Byte, 8 cylinder
	DC.B	$09		;5900RPM High Byte, 8 cylinder
	DC.B	$09		;6000RPM High Byte, 8 cylinder
	DC.B	$09		;6100RPM High Byte, 8 cylinder
	DC.B	$09		;6200RPM High Byte, 8 cylinder
	DC.B	$09		;6300RPM High Byte, 8 cylinder
	DC.B	$09		;6400RPM High Byte, 8 cylinder
	DC.B	$09		;6500RPM High Byte, 8 cylinder
	DC.B	$08		;6600RPM High Byte, 8 cylinder
	DC.B	$08		;6700RPM High Byte, 8 cylinder
	DC.B	$08		;6800RPM High Byte, 8 cylinder
	DC.B	$08		;6900RPM High Byte, 8 cylinder
	DC.B	$08		;7000RPM High Byte, 8 cylinder
	DC.B	$08		;7100RPM High Byte, 8 cylinder
	DC.B	$08		;7200RPM High Byte, 8 cylinder
	DC.B	$08		;7300RPM High Byte, 8 cylinder
	DC.B	$07		;7400RPM High Byte, 8 cylinder
	DC.B	$07		;7500RPM High Byte, 8 cylinder
	DC.B	$07		;7600RPM High Byte, 8 cylinder
	DC.B	$07		;7700RPM High Byte, 8 cylinder
	DC.B	$07		;7800RPM High Byte, 8 cylinder
	DC.B	$07		;7900RPM High Byte, 8 cylinder
	DC.B	$07		;8000RPM High Byte, 8 cylinder
	DC.B	$07		;8100RPM High Byte, 8 cylinder
	DC.B	$07		;8200RPM High Byte, 8 cylinder
	DC.B	$07		;8300RPM High Byte, 8 cylinder
	DC.B	$06		;8400RPM High Byte, 8 cylinder
	DC.B	$06		;8500RPM High Byte, 8 cylinder
	DC.B	$06		;8600RPM High Byte, 8 cylinder
	DC.B	$06		;8700RPM High Byte, 8 cylinder
	DC.B	$06		;8800RPM High Byte, 8 cylinder
	DC.B	$06		;8900RPM High Byte, 8 cylinder
	DC.B	$06		;9000RPM High Byte, 8 cylinder
	DC.B	$06		;9100RPM High Byte, 8 cylinder
	DC.B	$06		;9200RPM High Byte, 8 cylinder
	DC.B	$06		;9300RPM High Byte, 8 cylinder
	DC.B	$06		;9400RPM High Byte, 8 cylinder
	DC.B	$06		;9500RPM High Byte, 8 cylinder
	DC.B	$06		;9600RPM High Byte, 8 cylinder
	DC.B	$06		;9700RPM High Byte, 8 cylinder
	DC.B	$05		;9800RPM High Byte, 8 cylinder
	DC.B	$05		;9900RPM High Byte, 8 cylinder
	DC.B	$05		;10000RPM High Byte, 8 cylinder
	
RPM_TICKS_LOW_8:
	DC.B	$30		;500RPM Low Byte, 8 cylinder
	DC.B	$A8		;600RPM Low Byte, 8 cylinder
	DC.B	$B5		;700RPM Low Byte, 8 cylinder
	DC.B	$3E		;800RPM Low Byte, 8 cylinder
	DC.B	$1B		;900RPM Low Byte, 8 cylinder
	DC.B	$98		;1000RPM Low Byte, 8 cylinder
	DC.B	$44		;1100RPM Low Byte, 8 cylinder
	DC.B	$D4		;1200RPM Low Byte, 8 cylinder
	DC.B	$12		;1300RPM Low Byte, 8 cylinder
	DC.B	$DA		;1400RPM Low Byte, 8 cylinder
	DC.B	$10		;1500RPM Low Byte, 8 cylinder
	DC.B	$9F		;1600RPM Low Byte, 8 cylinder
	DC.B	$78		;1700RPM Low Byte, 8 cylinder
	DC.B	$8D		;1800RPM Low Byte, 8 cylinder
	DC.B	$D7		;1900RPM Low Byte, 8 cylinder
	DC.B	$4C		;2000RPM Low Byte, 8 cylinder
	DC.B	$E7		;2100RPM Low Byte, 8 cylinder
	DC.B	$A2		;2200RPM Low Byte, 8 cylinder
	DC.B	$7A		;2300RPM Low Byte, 8 cylinder
	DC.B	$6A		;2400RPM Low Byte, 8 cylinder
	DC.B	$70		;2500RPM Low Byte, 8 cylinder
	DC.B	$89		;2600RPM Low Byte, 8 cylinder
	DC.B	$B4		;2700RPM Low Byte, 8 cylinder
	DC.B	$ED		;2800RPM Low Byte, 8 cylinder
	DC.B	$34		;2900RPM Low Byte, 8 cylinder
	DC.B	$88		;3000RPM Low Byte, 8 cylinder
	DC.B	$E7		;3100RPM Low Byte, 8 cylinder
	DC.B	$50		;3200RPM Low Byte, 8 cylinder
	DC.B	$C1		;3300RPM Low Byte, 8 cylinder
	DC.B	$3C		;3400RPM Low Byte, 8 cylinder
	DC.B	$BE		;3500RPM Low Byte, 8 cylinder
	DC.B	$47		;3600RPM Low Byte, 8 cylinder
	DC.B	$D6		;3700RPM Low Byte, 8 cylinder
	DC.B	$6B		;3800RPM Low Byte, 8 cylinder
	DC.B	$06		;3900RPM Low Byte, 8 cylinder
	DC.B	$A6		;4000RPM Low Byte, 8 cylinder
	DC.B	$4B		;4100RPM Low Byte, 8 cylinder
	DC.B	$F3		;4200RPM Low Byte, 8 cylinder
	DC.B	$A0		;4300RPM Low Byte, 8 cylinder
	DC.B	$51		;4400RPM Low Byte, 8 cylinder
	DC.B	$05		;4500RPM Low Byte, 8 cylinder
	DC.B	$BD		;4600RPM Low Byte, 8 cylinder
	DC.B	$77		;4700RPM Low Byte, 8 cylinder
	DC.B	$35		;4800RPM Low Byte, 8 cylinder
	DC.B	$F5		;4900RPM Low Byte, 8 cylinder
	DC.B	$B8		;5000RPM Low Byte, 8 cylinder
	DC.B	$7D		;5100RPM Low Byte, 8 cylinder
	DC.B	$45		;5200RPM Low Byte, 8 cylinder
	DC.B	$0E		;5300RPM Low Byte, 8 cylinder
	DC.B	$DA		;5400RPM Low Byte, 8 cylinder
	DC.B	$A7		;5500RPM Low Byte, 8 cylinder
	DC.B	$77		;5600RPM Low Byte, 8 cylinder
	DC.B	$48		;5700RPM Low Byte, 8 cylinder
	DC.B	$1A		;5800RPM Low Byte, 8 cylinder
	DC.B	$EE		;5900RPM Low Byte, 8 cylinder
	DC.B	$C4		;6000RPM Low Byte, 8 cylinder
	DC.B	$9B		;6100RPM Low Byte, 8 cylinder
	DC.B	$73		;6200RPM Low Byte, 8 cylinder
	DC.B	$4D		;6300RPM Low Byte, 8 cylinder
	DC.B	$28		;6400RPM Low Byte, 8 cylinder
	DC.B	$04		;6500RPM Low Byte, 8 cylinder
	DC.B	$E1		;6600RPM Low Byte, 8 cylinder
	DC.B	$BF		;6700RPM Low Byte, 8 cylinder
	DC.B	$9E		;6800RPM Low Byte, 8 cylinder
	DC.B	$7E		;6900RPM Low Byte, 8 cylinder
	DC.B	$5F		;7000RPM Low Byte, 8 cylinder
	DC.B	$41		;7100RPM Low Byte, 8 cylinder
	DC.B	$23		;7200RPM Low Byte, 8 cylinder
	DC.B	$07		;7300RPM Low Byte, 8 cylinder
	DC.B	$EB		;7400RPM Low Byte, 8 cylinder
	DC.B	$D0		;7500RPM Low Byte, 8 cylinder
	DC.B	$B6		;7600RPM Low Byte, 8 cylinder
	DC.B	$9C		;7700RPM Low Byte, 8 cylinder
	DC.B	$83		;7800RPM Low Byte, 8 cylinder
	DC.B	$6B		;7900RPM Low Byte, 8 cylinder
	DC.B	$53		;8000RPM Low Byte, 8 cylinder
	DC.B	$3C		;8100RPM Low Byte, 8 cylinder
	DC.B	$25		;8200RPM Low Byte, 8 cylinder
	DC.B	$0F		;8300RPM Low Byte, 8 cylinder
	DC.B	$FA		;8400RPM Low Byte, 8 cylinder
	DC.B	$E5		;8500RPM Low Byte, 8 cylinder
	DC.B	$D0		;8600RPM Low Byte, 8 cylinder
	DC.B	$BC		;8700RPM Low Byte, 8 cylinder
	DC.B	$A9		;8800RPM Low Byte, 8 cylinder
	DC.B	$95		;8900RPM Low Byte, 8 cylinder
	DC.B	$83		;9000RPM Low Byte, 8 cylinder
	DC.B	$70		;9100RPM Low Byte, 8 cylinder
	DC.B	$5E		;9200RPM Low Byte, 8 cylinder
	DC.B	$4D		;9300RPM Low Byte, 8 cylinder
	DC.B	$3C		;9400RPM Low Byte, 8 cylinder
	DC.B	$2B		;9500RPM Low Byte, 8 cylinder
	DC.B	$1B		;9600RPM Low Byte, 8 cylinder
	DC.B	$0A		;9700RPM Low Byte, 8 cylinder
	DC.B	$FB		;9800RPM Low Byte, 8 cylinder
	DC.B	$EB		;9900RPM Low Byte, 8 cylinder
	DC.B	$DC		;10000RPM Low Byte, 8 cylinder
	

ADVANCE_HIGHB:
	DC.B	$06		;  0 deg
	DC.B	$05		;  1 deg
	DC.B	$05		;  2 deg
	DC.B	$05		;  3 deg
	DC.B	$05		;  4 deg
	DC.B	$05		;  5 deg
	DC.B	$05		;  6 deg
	DC.B	$05		;  7 deg
	DC.B	$05		;  8 deg
	DC.B	$05		;  9 deg
	DC.B	$05		; 10 deg
	DC.B	$04		; 11 deg
	DC.B	$04		; 12 deg
	DC.B	$04		; 13 deg
	DC.B	$04		; 14 deg
	DC.B	$04		; 15 deg
	DC.B	$04		; 16 deg
	DC.B	$04		; 17 deg
	DC.B	$04		; 18 deg
	DC.B	$04		; 19 deg
	DC.B	$04		; 20 deg
	DC.B	$03		; 21 deg
	DC.B	$03		; 22 deg
	DC.B	$03		; 23 deg
	DC.B	$03		; 24 deg
	DC.B	$03		; 25 deg
	DC.B	$03		; 26 deg
	DC.B	$03		; 27 deg
	DC.B	$03		; 28 deg
	DC.B	$03		; 29 deg
	DC.B	$03		; 30 deg
	DC.B	$02		; 31 deg
	DC.B	$02		; 32 deg
	DC.B	$02		; 33 deg
	DC.B	$02		; 34 deg
	DC.B	$02		; 35 deg
	DC.B	$02		; 36 deg
	DC.B	$02		; 37 deg
	DC.B	$02		; 38 deg
	DC.B	$02		; 39 deg
	DC.B	$02		; 40 deg
	DC.B	$01		; 41 deg
	DC.B	$01		; 42 deg
	DC.B	$01		; 43 deg
	DC.B	$01		; 44 deg
	DC.B	$01		; 45 deg
	DC.B	$01		; 46 deg
	DC.B	$01		; 47 deg
	DC.B	$01		; 48 deg
	DC.B	$01		; 49 deg
	DC.B	$01		; 50 deg
	DC.B	$00		; 51 deg
	DC.B	$00		; 52 deg
	DC.B	$00		; 53 deg
	DC.B	$00		; 54 deg
	DC.B	$00		; 55 deg
	DC.B	$00		; 56 deg
	DC.B	$00		; 57 deg
	DC.B	$00		; 58 deg
	DC.B	$00		; 59 deg
	
ADVANCE_LOWB:
	DC.B	$00		;  0 deg
	DC.B	$E6		;  1 deg
	DC.B	$CD		;  2 deg
	DC.B	$B3		;  3 deg
	DC.B	$9A		;  4 deg
	DC.B	$80		;  5 deg
	DC.B	$66		;  6 deg
	DC.B	$4D		;  7 deg
	DC.B	$33		;  8 deg
	DC.B	$1A		;  9 deg
	DC.B	$00		; 10 deg
	DC.B	$E6		; 11 deg
	DC.B	$CD		; 12 deg
	DC.B	$B3		; 13 deg
	DC.B	$9A		; 14 deg
	DC.B	$80		; 15 deg
	DC.B	$66		; 16 deg
	DC.B	$4D		; 17 deg
	DC.B	$33		; 18 deg
	DC.B	$1A		; 19 deg
	DC.B	$00		; 20 deg
	DC.B	$E6		; 21 deg
	DC.B	$CD		; 22 deg
	DC.B	$B3		; 23 deg
	DC.B	$9A		; 24 deg
	DC.B	$80		; 25 deg
	DC.B	$66		; 26 deg
	DC.B	$4D		; 27 deg
	DC.B	$33		; 28 deg
	DC.B	$1A		; 29 deg
	DC.B	$00		; 30 deg
	DC.B	$E6		; 31 deg
	DC.B	$CD		; 32 deg
	DC.B	$B3		; 33 deg
	DC.B	$9A		; 34 deg
	DC.B	$80		; 35 deg
	DC.B	$66		; 36 deg
	DC.B	$4D		; 37 deg
	DC.B	$33		; 38 deg
	DC.B	$1A		; 39 deg
	DC.B	$00		; 40 deg
	DC.B	$E6		; 41 deg
	DC.B	$CD		; 42 deg
	DC.B	$B3		; 43 deg
	DC.B	$9A		; 44 deg
	DC.B	$80		; 45 deg
	DC.B	$66		; 46 deg
	DC.B	$4D		; 47 deg
	DC.B	$33		; 48 deg
	DC.B	$1A		; 49 deg
	DC.B	$00		; 50 deg
	DC.B	$E6		; 51 deg
	DC.B	$CD		; 52 deg
	DC.B	$B3		; 53 deg
	DC.B	$9A		; 54 deg
	DC.B	$80		; 55 deg
	DC.B	$66		; 56 deg
	DC.B	$4D		; 57 deg
	DC.B	$33		; 58 deg
	DC.B	$1A		; 59 deg

KPAFACTOR:
	DC.B	100T
	DC.B	10T
	DC.B	11T
	DC.B	12T
	DC.B	13T
	DC.B	14T
	DC.B	15T
	DC.B	16T
	DC.B	17T
	DC.B	18T
	DC.B	19T
	DC.B	20T
	DC.B	21T
	DC.B	22T
	DC.B	23T
	DC.B	24T
	DC.B	25T
	DC.B	26T
	DC.B	27T
	DC.B	28T
	DC.B	29T
	DC.B	30T
	DC.B	31T
	DC.B	32T
	DC.B	33T
	DC.B	34T
	DC.B	35T
	DC.B	36T
	DC.B	37T
	DC.B	38T
	DC.B	39T
	DC.B	40T
	DC.B	41T
	DC.B	42T
	DC.B	43T
	DC.B	44T
	DC.B	45T
	DC.B	46T
	DC.B	47T
	DC.B	48T
	DC.B	49T
	DC.B	50T
	DC.B	51T
	DC.B	52T
	DC.B	53T
	DC.B	54T
	DC.B	55T
	DC.B	56T
	DC.B	57T
	DC.B	58T
	DC.B	59T
	DC.B	59T
	DC.B	60T
	DC.B	61T
	DC.B	62T
	DC.B	63T
	DC.B	64T
	DC.B	65T
	DC.B	66T
	DC.B	67T
	DC.B	68T
	DC.B	69T
	DC.B	70T
	DC.B	71T
	DC.B	72T
	DC.B	73T
	DC.B	74T
	DC.B	75T
	DC.B	76T
	DC.B	77T
	DC.B	78T
	DC.B	79T
	DC.B	80T
	DC.B	81T
	DC.B	82T
	DC.B	83T
	DC.B	84T
	DC.B	85T
	DC.B	86T
	DC.B	87T
	DC.B	88T
	DC.B	89T
	DC.B	90T
	DC.B	91T
	DC.B	92T
	DC.B	93T
	DC.B	94T
	DC.B	95T
	DC.B	96T
	DC.B	97T
	DC.B	98T
	DC.B	99T
	DC.B	100T
	DC.B	101T
	DC.B	102T
	DC.B	103T
	DC.B	104T
	DC.B	105T
	DC.B	106T
	DC.B	107T
	DC.B	108T
	DC.B	109T
	DC.B	110T
	DC.B	110T
	DC.B	111T
	DC.B	112T
	DC.B	113T
	DC.B	114T
	DC.B	115T
	DC.B	116T
	DC.B	117T
	DC.B	118T
	DC.B	119T
	DC.B	120T
	DC.B	121T
	DC.B	122T
	DC.B	123T
	DC.B	124T
	DC.B	125T
	DC.B	126T
	DC.B	127T
	DC.B	128T
	DC.B	129T
	DC.B	130T
	DC.B	131T
	DC.B	132T
	DC.B	133T
	DC.B	134T
	DC.B	135T
	DC.B	136T
	DC.B	137T
	DC.B	138T
	DC.B	139T
	DC.B	140T
	DC.B	141T
	DC.B	142T
	DC.B	143T
	DC.B	144T
	DC.B	145T
	DC.B	146T
	DC.B	147T
	DC.B	148T
	DC.B	149T
	DC.B	150T
	DC.B	151T
	DC.B	152T
	DC.B	153T
	DC.B	154T
	DC.B	155T
	DC.B	156T
	DC.B	157T
	DC.B	158T
	DC.B	159T
	DC.B	160T
	DC.B	160T
	DC.B	161T
	DC.B	162T
	DC.B	163T
	DC.B	164T
	DC.B	165T
	DC.B	166T
	DC.B	167T
	DC.B	168T
	DC.B	169T
	DC.B	170T
	DC.B	171T
	DC.B	172T
	DC.B	173T
	DC.B	174T
	DC.B	175T
	DC.B	176T
	DC.B	177T
	DC.B	178T
	DC.B	179T
	DC.B	180T
	DC.B	181T
	DC.B	182T
	DC.B	183T
	DC.B	184T
	DC.B	185T
	DC.B	186T
	DC.B	187T
	DC.B	188T
	DC.B	189T
	DC.B	190T
	DC.B	191T
	DC.B	192T
	DC.B	193T
	DC.B	194T
	DC.B	195T
	DC.B	196T
	DC.B	197T
	DC.B	198T
	DC.B	199T
	DC.B	200T
	DC.B	201T
	DC.B	202T
	DC.B	203T
	DC.B	204T
	DC.B	205T
	DC.B	206T
	DC.B	207T
	DC.B	208T
	DC.B	209T
	DC.B	210T
	DC.B	210T
	DC.B	211T
	DC.B	212T
	DC.B	213T
	DC.B	214T
	DC.B	215T
	DC.B	216T
	DC.B	217T
	DC.B	218T
	DC.B	219T
	DC.B	220T
	DC.B	221T
	DC.B	222T
	DC.B	223T
	DC.B	224T
	DC.B	225T
	DC.B	226T
	DC.B	227T
	DC.B	228T
	DC.B	229T
	DC.B	230T
	DC.B	231T
	DC.B	232T
	DC.B	233T
	DC.B	234T
	DC.B	235T
	DC.B	236T
	DC.B	237T
	DC.B	238T
	DC.B	239T
	DC.B	240T
	DC.B	241T
	DC.B	242T
	DC.B	243T
	DC.B	244T
	DC.B	245T
	DC.B	246T
	DC.B	247T
	DC.B	248T
	DC.B	249T
	DC.B	250T
	DC.B	251T
	DC.B	252T
	DC.B	253T
	DC.B	254T
	DC.B	255T
	DC.B	255T
	DC.B	255T
	DC.B	255T
	DC.B	255T
	DC.B	100T
	


;*********************************************
;PROGRAMMABLE DATA AREA
;*********************************************
        
;The RPM Bins represent the number of microseconds elapsed between each PIP signal
;4 Cylinder = 2 PIP signals per engine crank revolution
;6 cylinder = 3 PIP signals per engine crank revolution
;8 cylinder = 4 PIP signals per engine crank revolution

;4 cylinder: uS between PIP signals = 1000000 * ((1/(RPM/60))/2)
;6 cylinder: uS between PIP signals = 1000000 * ((1/(RPM/60))/3)
;8 cylinder: uS between PIP signals = 1000000 * ((1/(RPM/60))/4)

;The controller will be pre-programmed by the host PC with the appropriate microseconds
;between PIP periods, pre-targeting the appropriate number of cylinders. In other words; during
;runtime, the EDIS controller doesn't know how many cylinders the engine has; it only considers
;the time between PIP signals.

;Note this example targets a 4 cylinder setup!

	org 	$FC00
SAVED_CONFIG_DATA:
SAVED_RPMBINS
	DC.B	10T		; 1000 RPM
	DC.B	20T		; 2000 RPM
	DC.B	30T		; 3000 RPM
	DC.B	40T		; 4000 RPM
	DC.B	45T		; 4500 RPM
	DC.B	50T		; 5000 RPM
	DC.B	55T		; 5500 RPM
	DC.B	60T		; 6000 RPM
	DC.B	65T		; 6500 RPM
	DC.B	70T		; 7000 RPM

;Using a MPX4250 pressure transducer, the Kpa values
;will map 1:1 to the values converted by the Analog-Digital Converter.					
SAVED_MAPBINS:
	DC.B	20T		; 20 Kpa
	DC.B	50T		; 50 Kpa
	DC.B	60T		; 60 Kpa
	DC.B	70T		; 70 Kpa
	DC.B	100T	; 100 Kpa
	DC.B	135T	; 135 Kpa
	DC.B	160T	; 160 Kpa
	DC.B	180T	; 180 Kpa
	DC.B	200T	; 200 Kpa
	DC.B	230T	; 230 Kpa

;Ignition Map is RPM across the X axis; MAP value down the Y axis.
;The values are the actual ignition advance values in degrees.
SAVED_IGNITION_MAP:
	DC.B 	12T, 20T, 25T, 27T, 30T, 32T, 35T, 36T, 36T, 36T
	DC.B 	12T, 20T, 25T, 27T, 30T, 32T, 35T, 36T, 36T, 36T
	DC.B 	15T, 20T, 25T, 27T, 30T, 32T, 35T, 36T, 36T, 36T
	DC.B 	15T, 20T, 25T, 27T, 30T, 32T, 35T, 36T, 36T, 36T
	DC.B 	15T, 20T, 25T, 27T, 28T, 29T, 30T, 33T, 34T, 34T
	DC.B 	12T, 20T, 23T, 23T, 23T, 25T, 25T, 25T, 25T, 25T
	DC.B 	12T, 18T, 20T, 22T, 22T, 23T, 23T, 23T, 24T, 24T
	DC.B 	10T, 15T, 20T, 22T, 22T, 22T, 22T, 22T, 22T, 22T
	DC.B 	10T, 15T, 20T, 22T, 22T, 22T, 22T, 22T, 22T, 22T
	DC.B 	10T, 15T, 20T, 22T, 22T, 22T, 22T, 22T, 22T, 22T

SAVED_CONFIG_CYLINDERS:
	DC.B	$04					;number of cylinders - 4,6 or 8

SAVED_USER_OUTPUT_CONFIG:		;bits 0-3 indicate output type- 1 = RPM switch, 0 = MAP switch
	DC.B	$FF					;bits 4-7 indicate output mode- 1 = "on" when configured limit is exceeded, 0 = "off" when configured limit is exceeded
	
SAVED_USER_OUTPUT_1:
	DC.B	10T
SAVED_USER_OUTPUT_2:
	DC.B	12T
SAVED_USER_OUTPUT_3:
	DC.B	13T
SAVED_USER_OUTPUT_4:
	DC.B	14T
SAVED_REV_LIMITER:		;Rev limiter RPM
	DC.B	76T
SAVED_RPM_SHIFT_LIGHT:
	DC.B	70T			;shift light RPM

SAVED_CONFIG_DATA_END:

;*********************************************
;END PROGRAMMABLE DATA AREA
;*********************************************

; disabled to allow bootloader to take over managment of this
;	org	FLBPR		; protect vector block from flash erase routine (see AN1831)
;	DC.B	$F8
END  
